#!/usr/bin/env python

"""
Compare two segmentations
"""

from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

import sys

from . import log, Segmentation, add_common_options

def bases_in_segments(segments):
    """Return the number of bases in a segment array"""
    if segments is None or len(segments) == 0:
        return 0
    else:
        return (segments['end'] - segments['start']).sum()

def edit_distance(bedfile1, bedfile2, quick=False, verbose=True):
    """Given two segmentations, returns the edit distance (bp) between them"""
    segmentations = [Segmentation(bedfile, verbose=verbose)
                     for bedfile in [bedfile1, bedfile2]]

    for segmentation in segmentations:
        for segments in segmentation.chromosomes.itervalues():
            if not (segments['start'][1:] >= segments['end'][:-1]).all():
                raise SyntaxError("Overlapping segments found in: %s" % \
                                      segmentation.name)

    chroms = set()
    for segmentation in segmentations:
        chroms.update(segmentation.chromosomes.keys())

    labels = [segmentation.labels for segmentation in segmentations]

    bases_diff = 0
    bases_same = 0
    bases_missing1 = 0
    bases_missing2 = 0
    for chrom in chroms:
        log("%s" % chrom, verbose)

        # If no segments in segmentation 1
        try:
            segs1 = segmentations[0].chromosomes[chrom]
        except KeyError:
            segs1 = None


        # If no segments in segmentation 2
        try:
            segs2 = segmentations[1].chromosomes[chrom]
        except KeyError:
            segs2 = None

        if segs1 is None and segs2 is None:
            continue
        elif segs1 is None or segs2 is None:
            if segs1 is None:
                bases_missing1 += bases_in_segments(segs2)
            elif segs2 is None:
                bases_missing2 += bases_in_segments(segs1)
            continue

        # Segments in both segmentations
        segs1_iter = iter(segs1)
        start1, end1, key1 = segs1_iter.next()
        segs2_iter = iter(segs2)
        start2, end2, key2 = segs2_iter.next()
        while True:
            advance1 = False
            advance2 = False
            if start1 < start2:  # move up segment 1
                stop = min(start2, end1)
                bases_missing2 += stop - start1
                start1 = stop
                if end1 <= start2:
                    advance1 = True
            elif start2 < start1:  # move up segment 2
                stop = min(start1, end2)
                bases_missing1 += stop - start2
                start2 = stop
                if end2 <= start1:
                    advance2 = True
            else:  # start1 == start2
                if end1 < end2:
                    bases = end1 - start1
                    stop = end1
                    advance1 = True
                elif end2 < end1:
                    bases = end2 - start2
                    stop = end2
                    advance2 = True
                else:  # Segments match perfectly
                    bases = end2 - start2
                    stop = end2
                    advance1 = True
                    advance2 = True

                if labels[0][key1] == labels[1][key2]:
                    bases_same += bases
                else:
                    bases_diff += bases

                # Advance both
                start1 = stop
                start2 = stop

            # Carry out any pending advances
            if advance1:
                try:
                    start1, end1, key1 = segs1_iter.next()
                except StopIteration:
                    bases_missing1 += end2 - start2
                    for start2, end2, key2 in segs2_iter:
                        bases_missing1 += end2 - start2
                    break

            if advance2:
                try:
                    start2, end2, key2 = segs2_iter.next()
                except StopIteration:
                    bases_missing2 += end1 - start1
                    for start1, end1, key1 in segs1_iter:
                        bases_missing2 += end1 - start1
                    break

        if quick: break

    return bases_same, bases_diff, bases_missing1, bases_missing2

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTIONS] SEGMENTATION SEGMENTATION"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version,
                          description=__doc__.strip())

    parser.add_option("-d", "--edit-distance", dest="edit_distance",
                      default=False, action="store_true",
                      help="Measure the base-wise edit distance between"
                      " the two specified segmentations")
    add_common_options(parser, ['quick', 'quiet'])
    (options, args) = parser.parse_args(args)

    if len(args) < 2:
        parser.error("Insufficient number of arguments")

    return (options, args)

def print_edit_distance(*args):
    labels = ["bases the same",
              "bases different",
              "bases missing in first file",
              "bases missing in second file"]
    values = [int(val) for val in args]
    log("\n===== EDIT DISTANCE =====")
    for label, value in zip(labels, values):
        log("%s: \t%s" % (label, value))


## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfiles = args[0:2]
    if options.edit_distance:
        kwargs = dict(options.__dict__)
        results = edit_distance(*bedfiles, **kwargs)
        print_edit_distance(*results)
    else:
        log("No actions were specified")

if __name__ == "__main__":
    sys.exit(main())



#!/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 320 $"

"""
mnemonics.py

"""

import os

from . import log, RInterface
from .common import check_clobber

def create_mnemonic_file(datafile, dirpath, clobber=False,
                         namebase=None, gmtk=False, verbose=True):
    """Generate a mnemonic file with R clustering code.

    Datafile can either be a signal stats file or a GMTK params file
      (in which case gmtk=True must be specified to make.mnemonic.file)
    Calls R code that writes mnemonics to a file.
    Returns name of created mnemonic file
    """
    assert os.path.isfile(datafile)

    R = RInterface(["common.R", "track_statistics.R"], verbose=verbose)

    if namebase is None:
        namebase = os.path.basename(datafile)
        if namebase.endswith(".tab"):
            namebase = namebase[:-4]
        if namebase.endswith(".params"):
            namebase = namebase[:-7]

    mnemonic_base = os.extsep.join([namebase, "mnemonics"])

    filename = os.path.join(dirpath, mnemonic_base)
    check_clobber(filename, clobber)

    # Create mnemonic file via R
    try:
        r_filename = R.call("make.mnemonic.file",
                            datafile, filename, gmtk=gmtk)
        # Peel off rpy2 layers
        mnemonicfilename = str(r_filename[0])

        log("Created mnemonic file: %s" % mnemonicfilename, verbose)

        return mnemonicfilename
    except R.RError:
        log("ERROR: Failed to create mnemonic file."
            " Continuing without mnemonics.")
        return None

if __name__ == "__main__":
    pass



#!/usr/bin/env python
from __future__ import division, with_statement

"""
Provides command-line and package entry points for analyzing the
length distribution of entries in a provided segmentation or annotation file.

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE="length_distribution"

__version__ = "$Revision: 339 $"

import os
import sys

from collections import defaultdict
from numpy import concatenate, median

from . import Annotation, die, RInterface, open_transcript, \
     add_common_options
from .common import get_ordered_labels, LABEL_ALL, make_tabfilename, \
     setup_directory, tab_saver

from .html import save_html_div

FIELDNAMES_SUMMARY = ["label", "num.segs", "mean.len", "median.len",
                     "stdev.len", "num.bp", "frac.bp"]
FIELDNAMES = ["label", "length"]
NAMEBASE = "%s" % MODULE
NAMEBASE_SIZES = "segment_sizes"
TEMPLATE_FILENAME = "length_div.tmpl"

HTML_TITLE = "Length distribution"

PNG_WIDTH = 600
PNG_HEIGHT_BASE = 100  # Axes and label
PNG_HEIGHT_PER_LABEL = 45

R = RInterface(["common.R", "length.R"])

## Given an annotation file, returns the length of each entry
def annotation_lengths(annotation):
    # key: label_key
    # val: list of numpy.ndarray
    lengths = defaultdict(list)
    labels = annotation.labels

    # convert segment coords to lengths
    for rows in annotation.chromosomes.itervalues():
        for label_key in labels.iterkeys():
            labeled_row_indexes = (rows['key'] == label_key)
            labeled_rows = rows[labeled_row_indexes]
            length = labeled_rows['end'] - labeled_rows['start']
            lengths[label_key].append(length)
            lengths[LABEL_ALL].append(length)

    # key: label_key
    # val: int
    num_bp = {}

    # convert lengths to:
    # key: label_key
    # val: numpy.ndarray(dtype=integer)
    for label_key, label_lengths_list in lengths.iteritems():
        label_lengths = concatenate(label_lengths_list)
        lengths[label_key] = label_lengths
        num_bp[label_key] = label_lengths.sum()

    return lengths, num_bp

## Specifies the format of a row of the summary tab file
def make_size_row(label, lengths, num_bp, total_bp):
    return {"label": label,
            "num.segs": len(lengths),
            "mean.len": "%.3f" % lengths.mean(),
            "median.len": "%.3f" % median(lengths),
            "stdev.len": "%.3f" % lengths.std(),
            "num.bp": num_bp,
            "frac.bp": "%.3f" % (num_bp / total_bp)}

## Saves the length summary data to a tab file, using mnemonics if specified
def save_size_tab(lengths, labels, num_bp, dirpath, verbose=True,
                  namebase=NAMEBASE_SIZES, clobber=False):
    ordered_keys, labels = get_ordered_labels(labels)
    with tab_saver(dirpath, namebase, FIELDNAMES_SUMMARY,
                   clobber=clobber, verbose=verbose) as saver:
        # "all" row first
        total_bp = num_bp[LABEL_ALL]
        row = make_size_row(LABEL_ALL, lengths[LABEL_ALL],
                            total_bp, total_bp)
        saver.writerow(row)

        # Remaining label rows
        for label_key in ordered_keys:
            label = labels[label_key]
            label_lengths = lengths[label_key]
            num_bp_label = num_bp[label_key]
            row = make_size_row(label, label_lengths,
                                   num_bp_label, total_bp)
            saver.writerow(row)

def make_row(label, length):
    return {"label": label,
            "length": length}

def save_tab(lengths, labels, num_bp, dirpath, clobber=False, verbose=True):
    # fix order for iterating through dict; must be consistent
    label_keys = sorted(labels.keys())

    # XXXopt: allocating space in advance might be faster
    lengths_array = concatenate([lengths[label_key]
                                 for label_key in label_keys])

    # creates an array that has the label repeated as many times as
    # the length
    labels_array = concatenate([[labels[label_key]] * len(lengths[label_key])
                               for label_key in label_keys])

    with tab_saver(dirpath, NAMEBASE, FIELDNAMES, verbose=verbose,
                   clobber=clobber) as saver:
        for label, length in zip(labels_array, lengths_array):
            row = make_row(label, length)
            saver.writerow(row)

## Generates and saves an R plot of the length distributions
def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
              mnemonic_file=None, transcriptfile=None, ropts=None):
    R.start(transcriptfile=transcriptfile, verbose=verbose)

    # Load data from corresponding tab file
    tabfilename = make_tabfilename(dirpath, namebase)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.length", dirpath, namebase, tabfilename,
           mnemonic_file=mnemonic_file, clobber=clobber, ropts=ropts)

## Generates and saves an R plot of the length distributions
def save_size_plot(dirpath, namebase=NAMEBASE_SIZES, clobber=False,
                   verbose=True, mnemonic_file=None,
                   show_bases=True, show_segments=True,
                   transcriptfile=None, ropts=None):
    R.start(transcriptfile=transcriptfile, verbose=verbose)

    # Load data from corresponding tab file
    tabfilename = make_tabfilename(dirpath, namebase)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.segment.sizes", dirpath, namebase, tabfilename,
           mnemonic_file=mnemonic_file, clobber=clobber,
           show_segments=show_segments, show_bases=show_bases, ropts=ropts)

def save_html(dirpath, clobber=False, mnemonicfile=None, verbose=True):
    extra_namebases = {"sizes": NAMEBASE_SIZES}
    save_html_div(TEMPLATE_FILENAME, dirpath, namebase=NAMEBASE,
                  extra_namebases=extra_namebases, mnemonicfile=mnemonicfile,
                  tables={"": NAMEBASE_SIZES}, module=MODULE,
                  clobber=clobber, title=HTML_TITLE, verbose=verbose)

## Package entry point
def validate(filename, dirpath, clobber=False, replot=False, noplot=False,
             verbose=True, mnemonic_file=None, ropts=None,
             show_segments=True, show_bases=True):
    if not replot:
        setup_directory(dirpath)
        annotation = Annotation(filename, verbose=verbose)

        labels = annotation.labels

        lengths, num_bp = annotation_lengths(annotation)
        save_tab(lengths, labels, num_bp, dirpath,
                 clobber=clobber, verbose=verbose)
        save_size_tab(lengths, labels, num_bp, dirpath,
                      clobber=clobber, verbose=verbose)

    if not noplot:
        with open_transcript(dirpath, MODULE) as transcriptfile:
            save_plot(dirpath, mnemonic_file=mnemonic_file,
                      clobber=clobber, verbose=verbose,
                      transcriptfile=transcriptfile, ropts=ropts)
            save_size_plot(dirpath, clobber=clobber, verbose=verbose,
                           mnemonic_file=mnemonic_file,
                           show_segments=show_segments,
                           show_bases=show_bases, ropts=ropts,
                           transcriptfile=transcriptfile)

    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file,
              verbose=verbose)

def parse_options(args):
    from optparse import OptionGroup, OptionParser

    usage = "%prog [OPTIONS] ANNOTATION"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'replot', 'noplot'])
    group.add_option("--no-segments", action="store_false",
                     dest="show_segments", default=True,
                     help="Do not show total segments covered by labels"
                     " in size plot")
    group.add_option("--no-bases", action="store_false",
                     dest="show_bases", default=True,
                     help="Do not show total bases covered by labels"
                     " in size plot")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 1:
        parser.error("Inappropriate number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    filename = args[0]

    kwargs = dict(options.__dict__)
    outdir = kwargs.pop("outdir")
    validate(filename, outdir, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import with_statement

import inspect
import sys
import unittest

from numpy import array
from tempfile import NamedTemporaryFile

from segtools import Annotations, Segmentation
from segtools.overlap import calc_overlap

def data2bed(lines):
    # Input lines should each be a tuple of (chrom, start, end, label)
    strings = []
    for line in lines:
        strings.append("chr%s\t%d\t%d\t%s" % line)
    return "\n".join(strings)

def data2gff(lines):
    # Input lines should each be a tuple of (chrom, start, end, label)
    strings = []
    for line in lines:
        chr, start, end, label = line
        strings.append("chr%s\t%s\t%s\t%d\t%d" % \
                           (chr, "source", label, start+1, end))
    return "\n".join(strings)

class OverlapTester(unittest.TestCase):
    def init(self):
        pass

    def setUp(self):
        self.kwargs = {"verbose": False}
        self.init()

        # Set self.features from self.subject, self.query
        self.features = []
        self._open_files = []
        for type, data in [self.subject, self.query]:
            if type == "bed":
                new_file = NamedTemporaryFile(suffix=".bed")
                new_file.write(data2bed(data))
            elif type == "gff":
                new_file = NamedTemporaryFile(suffix=".gff")
                new_file.write(data2gff(data))
            elif type == "gtf":
                new_file = NamedTemporaryFile(suffix=".gtf")
                new_file.write(data2gff(data))

            new_file.flush()
            self._open_files.append(new_file)
            features = Annotations(new_file.name, verbose=False)

            if features:
                self.features.append(features)

    def tearDown(self):
        for file in self._open_files:
            file.close()

    def test(self):
        stats = calc_overlap(*self.features, **self.kwargs)
        self.assertValuesEqual(stats, self.answer)

    def assertArraysEqual(self, arr1, arr2):
        not_equal = (arr1 != arr2)
        if not_equal.any():
            self.fail("%r != %r" % (arr1, arr2))

    def assertValuesEqual(self, observed, expected):
        # counts, totals, nones
        for val1, val2 in zip(observed, expected):
            self.assertArraysEqual(val1, array(val2))

class TestSegmentPerfectOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 0, 50, 1),  # chrom, start, end, label
                                (1, 50, 100, 2)])
        self.query = ("bed", [(1, 0, 50, 1),
                              (1, 50, 100, 2)])
        self.answer = ([[1, 0], [0, 1]],  # subi vs qryj
                       (1, 1),  # total
                       (0, 0))  # none

class TestBasePerfectOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 0, 50, 1),
                                (1, 50, 100, 2)])
        self.query = ("bed", [(1, 0, 50, 1),
                              (1, 50, 100, 2)])
        self.answer = ([[50, 0], [0, 50]],
                       (50, 50),
                       (0, 0))

class TestSegmentNoOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 10, 20, 1)])
        self.query = ("bed", [(1, 20, 35, 6)])
        self.answer = ([0],
                       (1),
                       (1))

class TestBaseNoOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 10, 20, 1)])
        self.query = ("bed", [(1, 20, 35, 6)])
        self.answer = ([0],
                       (10),
                       (10))

class TestSegmentNoOverlapChrom(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 10, 20, 1)])
        self.query = ("bed", [(9, 20, 35, 6)])
        self.answer = ([0],
                       (1),
                       (1))

class TestBaseNoOverlapChrom(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 10, 20, 1)])
        self.query = ("bed", [(9, 20, 35, 6)])
        self.answer = ([0],
                       (10),
                       (10))

class TestSegmentSimpleOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 11, 20, 1),
                                (1, 25, 35, 2),
                                (1, 37, 40, 3)])
        self.query = ("gff", [(1, 15, 37, 2)])
        self.answer = ([[1], [1], [0]],
                       (1, 1, 1),
                       (0, 0, 1))

class TestBaseSimpleOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 11, 20, 1),
                                (1, 25, 35, 2),
                                (1, 37, 40, 3)])
        self.query = ("bed", [(1, 15, 37, 2)])
        self.answer = ([[5], [10], [0]],
                       (9, 10, 3),
                       (4, 0, 3))

class TestSegmentStackedFeatures(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 0, 10, 1)])
        self.query = ("gff", [(1, 2, 9, 2),
                              (1, 3, 12, 3),
                              (1, 4, 7, 4),
                              (1, 5, 6, 5)])
        self.answer = ([1, 1, 1, 1],
                       (1),
                       (0))

class TestBaseStackedFeatures(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 0, 10, 1)])
        self.query = ("gff", [(1, 2, 9, 2),
                              (1, 3, 12, 3),
                              (1, 4, 7, 4),
                              (1, 5, 6, 5)])
        self.answer = ([7, 7, 3, 1],
                       (10),
                       (2))

class TestSegmentOverlappingFeatures(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 0, 10, 1),
                                (1, 10, 20, 2),
                                (1, 20, 30, 1)])
        self.query = ("gff", [(1, 0, 30, 1),
                              (1, 2, 15, 1),
                              (1, 5, 20, 2),
                              (1, 10, 12, 1)])
        self.answer = ([[2, 1], [1, 1]],
                       (2, 1),
                       (0, 0))

class TestBaseOverlappingFeatures(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 0, 10, 1),
                                (1, 10, 20, 2),
                                (1, 20, 30, 1)])
        self.query = ("gff", [(1, 0, 30, 1),
                              (1, 2, 15, 1),
                              (1, 5, 20, 2),
                              (1, 10, 12, 1)])
        self.answer = ([[20, 5], [10, 10]],
                       (20, 10),
                       (0, 0))

class TestSegmentComplexOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 0, 4, 1),
                                (1, 10, 15, 2),
                                (1, 15, 21, 3),
                                (1, 26, 27, 1),
                                (1, 30, 31, 2),
                                (2, 14, 16, 2),
                                (3, 1, 100, 3)])
        self.query = ("bed", [(1, 0, 10, 1),
                              (1, 5, 15, 2),
                              (1, 15, 25, 1),
                              (1, 20, 30, 1),
                              (1, 20, 30, 2),
                              (1, 25, 30, 3),
                              (2, 5, 10, 1),
                              (2, 10, 15, 3)])
        self.answer = ([[2, 1, 1], [0, 1, 1], [1, 1, 0]],
                       (2, 3, 2),
                       (0, 1, 1))

class TestBaseComplexOverlap(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 0, 4, 1),
                                (1, 10, 15, 2),
                                (1, 15, 21, 3),
                                (1, 26, 27, 1),
                                (1, 30, 31, 2),
                                (2, 14, 16, 2),
                                (3, 1, 100, 3)])
        self.query = ("bed", [(1, 0, 10, 1),
                              (1, 5, 15, 2),
                              (1, 15, 25, 1),
                              (1, 20, 30, 1),
                              (1, 20, 30, 2),
                              (1, 25, 30, 3),
                              (2, 5, 10, 1),
                              (2, 10, 15, 3)])
        self.answer = ([[5, 1, 1], [0, 5, 1], [6, 1, 0]],
                       (5, 8, 105),
                       (0, 2, 99))

class TestSegmentComplexOverlapReversed(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "segments"
        self.subject = ("bed", [(1, 0, 10, 1),
                                (1, 5, 15, 2),
                                (1, 15, 25, 1),
                                (1, 20, 30, 1),
                                (1, 20, 30, 2),
                                (1, 25, 30, 3),
                                (2, 5, 10, 1),
                                (2, 10, 15, 3)])
        self.query = ("gff", [(1, 0, 4, 1),
                              (1, 10, 15, 2),
                              (1, 15, 21, 3),
                              (1, 26, 27, 1),
                              (1, 30, 31, 2),
                              (2, 14, 16, 2),
                              (3, 1, 100, 3)])
        self.answer = ([[2, 0, 2], [1, 1, 1], [1, 1, 0]],
                       (4, 2, 2),
                       (1, 0, 0))


class TestBaseComplexOverlapReversed(OverlapTester):
    def init(self):
        self.kwargs["mode"] = "bases"
        self.subject = ("bed", [(1, 0, 10, 1),
                                (1, 5, 15, 2),
                                (1, 15, 25, 1),
                                (1, 20, 30, 1),
                                (1, 20, 30, 2),
                                (1, 25, 30, 3),
                                (2, 5, 10, 1),
                                (2, 10, 15, 3)])
        self.query = ("gff", [(1, 0, 4, 1),
                              (1, 10, 15, 2),
                              (1, 15, 21, 3),
                              (1, 26, 27, 1),
                              (1, 30, 31, 2),
                              (2, 14, 16, 2),
                              (3, 1, 100, 3)])
        self.answer = ([[5, 0, 7], [1, 5, 1], [1, 1, 0]],
                       (35, 20, 10),
                       (23, 13, 8))

def suite():
    classes = []
    members = inspect.getmembers(sys.modules[__name__])
    for name, value in members:
        if inspect.isclass(value) and name.startswith("Test"):
            classes.append(value)

    tests = map(unittest.TestLoader().loadTestsFromTestCase, classes)
    return unittest.TestSuite(tests)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
Provides command-line and package entry points for analyzing the signal
distribution over tracks and labels.

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE="signal_distribution"

__version__ = "$Revision: 339 $"


import os
import sys

from collections import defaultdict
from genomedata import Genome
from itertools import repeat
from functools import partial
from numpy import apply_along_axis, array, ceil, compress, floor, fromiter, \
    histogram, isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF, square, zeros

from . import log, Segmentation, die, RInterface, add_common_options
from .common import iter_segments_continuous, iter_supercontig_segments, \
     make_tabfilename, setup_directory, tab_reader, tab_saver
from .html import save_html_div
from .mnemonics import create_mnemonic_file

FIELDNAMES = ["label", "trackname", "lower_edge", "count"]
NAMEBASE = "%s" % MODULE

FIELDNAMES_STATS = ["label", "trackname", "mean", "sd"]
NAMEBASE_STATS = os.extsep.join([NAMEBASE, "stats"])

HTML_TITLE = "Signal value distribution"
HTML_TEMPLATE_FILENAME = "signal_div.tmpl"

NBINS = 100

R = RInterface(["common.R", "signal.R"])

class SignalHistogram(object):
    def __init__(self, histogram=None, **fields):
        self._data = histogram
        self._metadata = fields

    @property
    def data(self):
        return self._data

    @property
    def metadata(self):
        return self._metadata

    @staticmethod
    def calculate(genome, segmentation, nbins=None, chroms=None,
                  calc_ranges=False,
                  value_range=(None, None), quick=False, verbose=True):
        """Computes a set of histograms, one for each track-label pair.

        if nbins is:
        - None: There will be a bin for every possible value
        - > 0: There will be this many bins

        if chroms is:
        - a sequence of chromosome names: only those chromosomes are processed
        - False, [], None: all chromosomes are processed

        if calc_ranges is:
        - False: use the precomputed ranges stored for the entire genome file
        - True: calculate the segmentation ranges across the entire input

        value_range is a (min, max) tuple:
          if either value is None, it is ignored
          if min/max is not None, it is used as a limit for all binning
          if both are specified, calc_ranges is ignored

        if quick is:
        - True: the histogram is calculated for only the first chromosome

        returns a dict

        key: label (not label_key)
        val: dict
          key: trackname
          val: [hist, edges]

        """
        assert genome is not None
        assert segmentation is not None

        labels = segmentation.labels

        tracks = genome.tracknames_continuous
        max_bins = ceil(genome.maxs).astype(int)
        min_bins = floor(genome.mins).astype(int)
        # A dict from tracks to a range tuple
        track_ranges = dict(zip(tracks, zip(min_bins, max_bins)))

        if len(tracks) == 0:
            die("Trying to calculate histogram for no tracks")

        # key: trackname
        # val: track_hist_func: a function that generates a histogram from data
        #                       with uniform bins within a track
        # For every trackname, there's a function that generates histograms
        track_hist_funcs = dict((trackname,
                                 partial(histogram,
                                         bins=xrange(min_bin, max_bin + 1)))
                                for trackname, (min_bin, max_bin) in
                                track_ranges.iteritems())
        # Trackname is one the track names, track_range is (min,max)
        # functools.partial(func[, *args][, **keywords])
        #   Returns a new partial object which when called will behave like
        #   func called with args and keywords.

        # Result dictionary: see function docstring for specification
        # Now for every trackname and for every label
        #   there's a histogram array of the right boundary and frequency
        res = dict((trackname, dict((label,
                                     list(track_hist_func(array([]))))
                                    for label in labels.itervalues()))
                   for trackname, track_hist_func in \
                       track_hist_funcs.iteritems())

        nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
        log("Generating signal distribution histograms", verbose)

        with genome:
            if chroms:
                chromosomes = [genome[chrom] for chrom in chroms]
            else:
                chromosomes = genome

            for chromosome in chromosomes:
                # Iterate through supercontigs and segments together
                for segment, continuous_seg in \
                        iter_segments_continuous(chromosome, segmentation,
                                                 verbose=verbose):
                    seg_label = labels[segment['key']]
                    # Iterate through each track
                    for trackname, track_hist_func in \
                            track_hist_funcs.iteritems():
                        # col_index is the index of the trackname
                        #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
                        col_index = chromosome.index_continuous(trackname)

                        # continuous_col is the "intensity"
                        #   (continuous number) from the data track
                        # len(supercontig_map) = len(continuous_col)
                        #   because they are for the same segment
                        continuous_col = continuous_seg[:, col_index]

                        # Remove the NaN's, otherwise numpy.histogram
                        #   gets confused and misses some of the data
                        cur_col_nonan = remove_nans(continuous_col)

                        # Keep track of number of real data values processed
                        if trackname in segmentation.tracks:
                            nseg_dps += cur_col_nonan.shape[0]

                        # Edges is a list with the left edge of every bin,
                        #   hist is the frequency for every bin
                        # This calls numpy.histogram
                        hist, edges = track_hist_func(cur_col_nonan)

                        cur_res = res[trackname][seg_label]
                        cur_res[0] += hist

                if quick: break  # 1 chromosome

        log("Segmentation tracks: %s\n" % str(segmentation.tracks), verbose)
        log("Read %s non-NaN values from segmentation tracks" % nseg_dps,
            verbose)

        return SignalHistogram(res, nseg_dps=nseg_dps)

    @staticmethod
    def calculate_stat(genome, segmentation, chroms=None,
                       quick=False, verbose=True):
        """
        Computes a mean and variance for each track-label pair.
        

        if chroms is:
        - a sequence of chromosome names: only those chromosomes are processed
        - False, [], None: all chromosomes are processed

        if calc_ranges is:
        - False: use the precomputed ranges stored for the entire genome file
        - True: calculate the segmentation ranges across the entire input
 
        if quick is:
        - True: the histogram is calculated for only the first chromosome
 
        returns a dict
 
        """
        assert genome is not None
        assert segmentation is not None
 
        labels = segmentation.labels
        
        tracks = genome.tracknames_continuous
        max_bins = ceil(genome.maxs).astype(int)
        min_bins = floor(genome.mins).astype(int)
        # A dict from tracks to a range tuple
        track_indices = dict(zip(tracks, range(len(tracks))))
        
        if len(tracks) == 0:
            die("Trying to calculate histogram for no tracks")
            
        nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
        sum_total, sum2_total, nseg_dps_total = zeros(\
            (len(tracks), len(labels)), dtype=float),  
        zeros((len(tracks), len(labels)), dtype=float), 
        zeros((len(tracks), len(labels)), dtype=float)
        log("Generating signal distribution histograms", verbose)
 
        with genome:
            if chroms:
                chromosomes = [genome[chrom] for chrom in chroms]
            else:
                chromosomes = genome
                
            for chromosome in chromosomes:
                # Iterate through supercontigs and segments together
                for segment, continuous_seg in \
                        iter_segments_continuous(chromosome, segmentation,
                                                 verbose=verbose):
 
                    sum_segment, sum2_segment, nseg_dps_segment  = \
                        apply_along_axis(get_stat, 0, continuous_seg)
                    seg_label = labels[segment['key']]
                    sum_total[:,seg_label], \
                        sum2_total[:,seg_label], \
                        nseg_dps_total[:,seg_label] = \
                        sum_total[:,seg_label] + sum_segment, \
                        sum2_total[:,seg_label] + sum2_segment, \
                        nseg_dps_total[:,seg_label] + nseg_dps_segment
                     # Iterate through each track
 
                if quick: break  # 1 chromosome
 
        mean = sum_total/nseg_dps_total
        sd = (sum2_total -
              ((sum_total**2)/nseg_dps_total))/(nseg_dps_total -1)
        nseg_dps = sum(nseg_dps_total)
        stats = defaultdict(partial(defaultdict, dict))
        for trackname, track_index in track_indices.iteritems():
            for label in labels:
                cur_stat = stats[label][trackname]
                cur_stat["sd"] = sd[track_index,label]
                cur_stat["mean"] = mean[track_index,label]
        return stats, nseg_dps 
 

    @staticmethod
    def calculate2(genome, segmentation, nbins=None, chroms=None,
                  calc_ranges=False,
                  value_range=(None, None), quick=False, verbose=True):
        assert genome is not None
        assert segmentation is not None

        labels = segmentation.labels

        tracks = genome.tracknames_continuous
        max_bins = ceil(genome.maxs).astype(int)
        min_bins = floor(genome.mins).astype(int)
        # A dict from tracks to a range tuple
        track_ranges = dict(zip(tracks, zip(min_bins, max_bins)))

        if len(tracks) == 0:
            die("Trying to calculate histogram for no tracks")

        # key: trackname
        # val: track_hist_func: a function that generates a histogram from data
        #                       with uniform bins within a track
        # For every trackname, there's a function that generates histograms
        track_hist_funcs = dict((trackname,
                                 partial(histogram,
                                         bins=xrange(min_bin, max_bin + 1)))
                                for trackname, (min_bin, max_bin) in
                                track_ranges.iteritems())
        # Trackname is one the track names, track_range is (min,max)
        # functools.partial(func[, *args][, **keywords])
        #   Returns a new partial object which when called will behave like
        #   func called with args and keywords.

        # Result dictionary: see function docstring for specification
        # Now for every trackname and for every label
        #   there's a histogram array of the right boundary and frequency
        res = dict((trackname, dict((label,
                                     list(track_hist_func(array([]))))
                                    for label in labels.itervalues()))
                   for trackname, track_hist_func in \
                       track_hist_funcs.iteritems())

        nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
        log("Generating signal distribution histograms", verbose)

        with genome:
            if chroms:
                chromosomes = [genome[chrom] for chrom in chroms]
            else:
                chromosomes = genome

            for chromosome in chromosomes:
                # Iterate through supercontigs and segments together
                for supercontig, segments in \
                        iter_supercontig_segments(chromosome, segmentation,
                                                  verbose=verbose):
                    continuous = supercontig.continuous
                    for key in labels:
                        seg_label = labels[key]
                        condition = segments['key'] == key
                        print "starting==========="
                        continuous_chunk = compress(condition, continuous,
                                                    axis=0)
                        print "done==========="
                        sys.exit(1)
                        #sums = nansum(continuous_chunk, axis=0)
                        #sum_sqs = nansum(power(continuous_chunk, 2.0), axis = 0)
                        for trackname, track_hist_func \
                                in track_hist_funcs.iteritems():
                            # col_index is the index of the trackname
                            #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
                            col_index = chromosome.index_continuous(trackname)

                            cur_col = continuous_chunk[:, col_index]
                            cur_col_nonan = cur_col[isfinite(cur_col)]
                            # Edges is a list with the left edge of every bin,
                            #   hist is the frequency for every bin
                            # This calls numpy.histogram
                            hist, edges = track_hist_func(cur_col_nonan)

                            cur_res = res[trackname][seg_label]
                            cur_res[0] += hist

                if quick: break  # 1 chromosome

        log("Segmentation tracks: %s\n" % str(segmentation.tracks), verbose)
        log("Read %s non-NaN values from segmentation tracks" % nseg_dps,
            verbose)

        return SignalHistogram(res, nseg_dps=nseg_dps)


    @staticmethod
    def read(dirpath, namebase=NAMEBASE, **kwargs):
        """Read a histogram from an output directory"""
        with tab_reader(dirpath, namebase, **kwargs) as (reader, metadata):
            histogram = {}
            try:
                nseg_dps = int(metadata["nseg_dps"])
            except KeyError:
                nseg_dps = 0

            for row in reader:
                type = row[0]
                if type == "track":
                    trackname = row[1]
                    edges = fromiter(row[2:], float)
                    histogram[trackname] = {}
                elif type == "label":
                    label = row[1]
                    hist = fromiter(row[2:], int)
                    histogram[trackname][label] = (hist, edges)
                else:
                    raise IOError("Unexpected row label: %s in input"
                                  " directory: %s" % (type, dirpath))

        return SignalHistogram(histogram, nseg_dps=nseg_dps)

    def save(self, dirpath, namebase=NAMEBASE, **kwargs):
        """Saves the histogram data to a tab file"""
        with tab_saver(dirpath, namebase, metadata=self.metadata,
                       **kwargs) as saver:
            for trackname in sorted(self.data.keys()):
                label_hists = self.data[trackname]
                first = True
                for label in sorted(label_hists.keys()):
                    (hist, edges) = label_hists[label]
                    if first:
                        nice_edges = edges.tolist()
                        saver.writerow(["track", trackname] + nice_edges)
                        first = False

                    nice_hist = hist.astype(int).tolist()
                    saver.writerow(["label", label] + nice_hist)

    def add(self, o_histogram):
        """Add data from a second histogram"""
        assert isinstance(o_histogram, SignalHistogram)
        if self.data is None:
            self._data = o_histogram.data
        else:
            for trackname, label_hists in self.data.iteritems():
                o_label_hists = o_histogram.data[trackname]
                for label, (hist, edges) in label_hists.iteritems():
                    o_hist, o_edges = o_label_hists[label]
                    assert (edges == o_edges).all()
                    hist += o_hist

def calc_stats(histogram):
    """Calculate track statistics (mean, sd) for each label

    histogram: a SignalHistogram

    Values are approximated from binned distributions
    - mean is a lower-bound on the actual mean

    Returns a dict: label_key -> dict( trackname -> {"mean", "sd", ...} )

    """
    stats = defaultdict(partial(defaultdict, dict))
    for trackname, label_hists in histogram.data.iteritems():
        for label, (hist, edges) in label_hists.iteritems():
            cur_stat = stats[label][trackname]
            n = nansum(hist)
            if n == 0:
                mean = NAN
                sd = NAN
            else:
                mean = (hist * edges[:-1]).sum() / n
                sd = (hist * (edges[:1] - mean)**2).sum() / (n - 1)

            cur_stat["sd"] = sd
            cur_stat["mean"] = mean

    return stats


## Saves the track stats to a tab file
def save_stats_tab(stats, dirpath, clobber=False, verbose=True,
                   namebase=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
    with tab_saver(dirpath, namebase, fieldnames, verbose=verbose,
                   clobber=clobber) as saver:
        for label, label_stats in stats.iteritems():
            for trackname, track_stats in label_stats.iteritems():
                mean = track_stats["mean"]
                sd = track_stats["sd"]
                saver.writerow(locals())

def save_stats_tab_genhist(stats, dirpath, clobber=False, verbose=True,
                           namebase=NAMEBASE_STATS, 
                           fieldnames=FIELDNAMES_STATS):
    with tab_saver(dirpath, namebase, fieldnames, verbose=verbose,
                   clobber=clobber) as saver:
        for label, label_stats in stats.iteritems():
            for trackname, track_stats in label_stats.iteritems():
                mean = track_stats["mean"]
                sd = track_stats["sd"]
                saver.writerow(locals())

def constant_factory(val):
    return repeat(val).next

def remove_nans(numbers):
    return numbers[isfinite(numbers)]

def get_stat(arr):
    arr_nonan = remove_nans(arr)
    return arr_nonan.sum(), square(arr_nonan).sum(), arr_nonan.shape[0]

## Returns a dict of trackname -> tuples: (min, max), one for each trackname
def seg_min_max(chromosome, segmentation, verbose=False):
    #print >>sys.stderr, "Looking for min & max in chromosome ", \
    #     chromosome.name, " for track ", track_index
    #print >>sys.stderr, "segmentation: ", segmentation.chromosomes

    tracknames = chromosome.tracknames_continuous
    limits = dict([(trackname, (PINF, NINF)) for trackname in tracknames])

    if chromosome.name not in segmentation.chromosomes.keys():
        return limits

    for segment, continuous in \
            iter_segments_continuous(chromosome, segmentation,
                                     verbose=verbose):
        if continuous.shape[0] > 0:
            for trackname in tracknames:
                col_index = chromosome.index_continuous(trackname)
                continuous_col = continuous[:, col_index]

                if len(continuous_col) != 0:
                    old_min, old_max = limits[trackname]
                    cur_min = nanmin(continuous_col, axis=0)
                    cur_max = nanmax(continuous_col, axis=0)
                    limits[trackname] = (min(old_min, cur_min),
                                         max(old_max, cur_max))

    return limits

## Loads the ranges of each track from the genomedata object
##   unless a segmentation is specified, in which case the ranges
##   are calculated for that segmentation
## Thus, defaults to loading ranges for all tracks from genomedata
def load_track_ranges(genome, segmentation=None):
    """
    returns a dict
    key: trackname
    val: (min, max) for that trackname
    """
    # start with the most extreme possible values
    res = defaultdict(constant_factory((PINF, NINF)))
    for chromosome in genome:
        if segmentation is not None:
            log("\t%s" % chromosome.name)
            res_chrom = seg_min_max(chromosome, segmentation)
            for trackname, limits in res_chrom.iteritems():
                old_min, old_max = res[trackname]
                cur_min, cur_max = limits
                res[trackname] = (min(old_min, cur_min), max(old_max, cur_max))
                #print >>sys.stderr, "Limits for %s: %s" % \
                #     (trackname, res[trackname])
        else:
            tracknames = chromosome.tracknames_continuous
            for trackname in tracknames:
                col_index = chromosome.index_continuous(trackname)
                cur_min = chromosome.mins[col_index]
                cur_max = chromosome.maxs[col_index]

                old_min, old_max = res[trackname]
                res[trackname] = (min(old_min, cur_min), max(old_max, cur_max))

    # Cast out of defaultdict
    return dict(res)


# ## Computes a set of histograms, one for each track-label pair.
# def calc_histogram2(genome, segmentation, quick=False, verbose=True,
#                    calc_ranges=False,
#                    value_range=(None, None), nbins=None,):
#     """
#     if quick is:
#     - True: the histogram is calculated for only the first chromosome

#     returns a dict:
#       key: trackname
#       val: list of [hist, label_edges, value_edges]:
#         hist: numpy.ndarray:
#           rows: labels [0, n]
#           cols: value_bins
#         label_edges: numpy.ndarray of label bins
#         value_edges: numpy.ndarray of value bins
#     """
#     assert genome is not None
#     assert segmentation is not None

#     labels = segmentation.labels

#     label_bins = xrange(0, len(labels) + 1)  # [0, 1, ..., n, n + 1]

#     tracks = genome.tracknames_continuous
#     max_bins = ceil(genome.maxs).astype(int)
#     min_bins = floor(genome.mins).astype(int)
#     # A dict from tracks to a range tuple
#     track_ranges = dict(zip(tracks, zip(min_bins, max_bins)))

#     # A dict of functions, one for each track that,
#     #   given (label_map, track_continuous) generates a
#     #   2d-histogram across labels and continuous values
#     track_hist_funcs = dict([(trackname,
#                               partial(histogram2d,
#                                       bins=[label_bins,
#                                             xrange(min_bin, max_bin + 1)]))
#                              for trackname, (min_bin, max_bin) in
#                              track_ranges.iteritems()])

#     # Return data: see function docstring for specification
#     # Initialize to appropriate size and type by calling functions with
#     #   values that won't be counted
#     res = {}
#     for trackname in tracks:
#         hist, xedges, yedges = track_hist_funcs[trackname]([PINF], [PINF])
#         res[trackname] = [hist.astype(int), xedges.astype(int),
#                           yedges.astype(int)]

#     print >>sys.stderr, "Generating signal distribution histograms"
#     ntracks = len(tracks)
#     nlabels = len(labels)
#     # Number of defined (non-NaN) data points found in segmentation tracks
#     nseg_dps = 0


#     nsupercontigs = 0
#     with genome:
#         for chromosome in genome:
#             for supercontig, segments in \
#                     iter_supercontig_segments(chromosome, segmentation,
#                                               verbose=False):
#                 nsupercontigs += 1

#     print "Created progress bar from %d items" % (nsupercontigs * ntracks)
#     progress = ProgressBar(nsupercontigs * ntracks)

#     with genome:
#         for chromosome in genome:
#             for supercontig, segments in \
#                     iter_supercontig_segments(chromosome, segmentation,
#                                               verbose=False):
#                 continuous = supercontig.continuous
#                 start_i = max(segments['start'][0], supercontig.start)
#                 end_i = min(segments['end'][-1], supercontig.end)
#                 label_map = map_segment_label(segments, (start_i, end_i))

#                 for trackname, track_hist_func in track_hist_funcs.iteritems():
#                     # col_index is the index of the trackname
#                     #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
#                     col_index = chromosome.index_continuous(trackname)

#                     # select track data within the range of the segments and
#                     #   for the specified track
#                     continuous_col = continuous[start_i:end_i, col_index]

#                     # Mask continuous_col and label_map to only indices
#                     #   where the data is defined
#                     keep_mask = isfinite(continuous_col)
#                     continuous_col_ok = continuous_col[keep_mask]
#                     label_map_ok = label_map[keep_mask]

#                     # Keep track of number of defined data points
#                     if trackname in segmentation.tracks:
#                         nseg_dps += continuous_col_ok.shape[0]

#                     # This calls numpy.histogram2d
#                     hist, xedges, yedges = track_hist_func(label_map_ok,
#                                                            continuous_col_ok)
#                     res_trackname = res[trackname]
#                     assert (res_trackname[1] == xedges).all()
#                     assert (res_trackname[2] == yedges).all()
#                     res_trackname[0] += hist.astype(int)

#                     progress.next()

#             if quick: break

#     progress.end()
#     print ""
#     print "Segmentation tracks: %s" % str(segmentation.tracks)
#     print "Read %s non-NaN values from segmentation tracks\n" % nseg_dps
#     return res, nseg_dps

# def calc_stats2(histogram):
#     """Calculate approximate track statistics (mean, sd) for each label

#     histogram: the histogram returned by calc_histogram

#     Values are approximated from binned distributions
#     - mean is a lower-bound on the actual mean

#     Returns a dict: label_key -> dict( trackname -> {"mean", "sd", ...} )

#     """
#     stats = defaultdict(partial(defaultdict, dict))
#     for trackname, track_hist_data in histogram.iteritems():
#         (track_hist, label_edges, value_edges) = track_hist_data
#         for label_edge, hist_row in zip(label_edges, track_hist):
#             label_key = int(label_edge)
#             cur_stat = stats[trackname][label_key]
#             n = nansum(hist_row)
#             if n == 0:
#                 mean = NAN
#                 sd = NAN
#             else:
#                 mean = (hist_row * value_edges[:-1]).sum() / n
#                 sd = (hist_row * (value_edges[:1] - mean)**2).sum() / (n - 1)

#             cur_stat["sd"] = sd
#             cur_stat["mean"] = mean

#     return stats

# ## Saves the histogram data to a tab file
# def save_tab(labels, histogram, dirpath, clobber=False,
#              namebase=NAMEBASE, fieldnames=FIELDNAMES):
#     with tab_saver(dirpath, namebase, fieldnames, clobber=clobber) as saver:
#         for label_key, label_histogram in histogram.iteritems():
#             label = labels[label_key]
#             for trackname, (hist, edges) in label_histogram.iteritems():
#                 for lower_edge, count in zip(edges, hist.tolist() + ["NA"]):
#                     saver.writerow(locals())


# ## Saves the track stats to a tab file
# def save_stats_tab2(labels, stats, dirpath, clobber=False,
#                    namebase=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
#     with tab_saver(dirpath, namebase, fieldnames, clobber=clobber) as saver:
#         for trackname, track_stats in stats.iteritems():
#             for label_key, label_stats in track_stats.iteritems():
#                 label = labels[label_key]
#                 mean = label_stats["mean"]
#                 sd = label_stats["sd"]
#                 saver.writerow(locals())

def save_stats_plot(dirpath, namebase=NAMEBASE_STATS, filename=None,
                    clobber=False, mnemonic_file=None, translation_file=None,
                    allow_regex=False, gmtk=False, verbose=True,
                    label_order=[], track_order=[], ropts=None):
    """
    if filename is specified, it overrides dirpath/namebase.tab as
    the data file for plotting.

    """
    ## Plot the track stats data with R
    R.start(verbose=verbose)
    R.source("track_statistics.R")

    if filename is None:
        filename = make_tabfilename(dirpath, namebase)

    if not os.path.isfile(filename):
        die("Unable to find stats data file: %s" % filename)

    R.plot("save.track.stats", dirpath, namebase, filename,
           mnemonic_file=mnemonic_file,
           translation_file=translation_file, ropts=ropts,
           as_regex=allow_regex, gmtk=gmtk, clobber=clobber,
           label_order=label_order, track_order=track_order)

def save_html(dirpath, genomedatadir, nseg_dps=None, verbose=True,
              ecdf=False, clobber=False):
    if ecdf:
        title = "%s (ECDF mode)" % HTML_TITLE
    else:
        title = HTML_TITLE

    title = "%s (%s)" % (title, os.path.basename(genomedatadir))

    if nseg_dps is None:
        nseg_dps = "???"

    extra_namebases = {"stats": NAMEBASE_STATS}
    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, title=title, segdatapoints=nseg_dps,
                  extra_namebases=extra_namebases, verbose=verbose)

def read_order_file(filename):
    if filename is None:
        return []
    elif os.path.isfile(filename):
        order = []
        with open(filename) as ifp:
            for line in ifp:
                line = line.strip()
                if line:
                    order.append(line)

        return order
    else:
        raise IOError("Could not find order file: %s" % filename)


## Package entry point
def validate(bedfilename, genomedatadir, dirpath,
             clobber=False, calc_ranges=False, inputdirs=None,
             quick=False, replot=False, noplot=False, verbose=True,
             nbins=NBINS, value_range=(None, None),
             ecdf=False, mnemonic_file=None, genhist=False,
             create_mnemonics=False, chroms=None, ropts=None,
             label_order_file=None, track_order_file=None):

    if not genhist or not replot:
        setup_directory(dirpath)
        genome = Genome(genomedatadir)
        segmentation = Segmentation(bedfilename, verbose=verbose)

        ntracks = genome.num_tracks_continuous  # All tracks; not just segtracks
        segtracks = segmentation.tracks
        labels = segmentation.labels

        nlabels = len(labels)
        fieldnames = FIELDNAMES


    if genhist:
        if inputdirs:
            histogram = SignalHistogram()
            for inputdir in inputdirs:
                try:
                    sub_histogram = SignalHistogram.read(inputdir, verbose=verbose)
                except IOError, e:
                    log("Problem reading data from %s: %s" % (inputdir, e))
                else:
                    histogram.add(sub_histogram)

        elif not replot:
            # Generate histogram
            histogram = SignalHistogram.calculate(genome, segmentation,
                                                  nbins=nbins, verbose=verbose,
                                                  calc_ranges=calc_ranges,
                                                  value_range=value_range,
                                                  quick=quick, chroms=chroms)
        else:
            histogram = SignalHistogram.read(dirpath, verbose=verbose)

        if not replot:
            histogram.save(dirpath, clobber=clobber, verbose=verbose)

            stats = calc_stats(histogram)
            save_stats_tab(stats, dirpath, clobber=clobber, verbose=verbose,
                           namebase=NAMEBASE_STATS)

            if mnemonic_file is None and create_mnemonics:
                statsfilename = make_tabfilename(dirpath, NAMEBASE_STATS)
                mnemonic_file = create_mnemonic_file(statsfilename, dirpath,
                                                     clobber=clobber,
                                                     verbose=verbose)
    else:
        stats, nseg_dps = SignalHistogram.calculate_stat(\
            genome, segmentation, quick=quick, chroms=chroms)

        save_stats_tab(stats, dirpath, clobber=clobber, verbose=verbose,
                       namebase=NAMEBASE_STATS)

        if mnemonic_file is None and create_mnemonics:
            statsfilename = make_tabfilename(dirpath, NAMEBASE_STATS)
            mnemonic_file = create_mnemonic_file(statsfilename, dirpath,
                                                 clobber=clobber,
                                                 verbose=verbose)

 
    if not noplot:
        if label_order_file is not None:
            log("Reading label ordering from: %s" % label_order_file)
        label_order = read_order_file(label_order_file)

        if track_order_file is not None:
            log("Reading track ordering from: %s" % track_order_file)
        track_order = read_order_file(track_order_file)

        save_stats_plot(dirpath, namebase=NAMEBASE_STATS, clobber=clobber,
                        mnemonic_file=mnemonic_file, verbose=verbose,
                        label_order=label_order, track_order=track_order,
                        ropts=ropts)

    if genhist and histogram:
        try:
            nseg_dps = histogram.metadata["nseg_dps"]
        except KeyError:
            nseg_dps = None

    if not genhist or histogram:
        save_html(dirpath, genomedatadir, nseg_dps=nseg_dps,
                  ecdf=ecdf, clobber=clobber, verbose=verbose)


def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] SEGMENTATION GENOMEDATAFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'quick', 'noplot',
                               'replot'])
    group.add_option("--genhist", action="store_true",
                     dest="genhist", default=False,
                     help="Generate histogram for each track")
    group.add_option("--create-mnemonics", action="store_true",
                     dest="create_mnemonics", default=False,
                     help="If mnemonics are not specified, they will be"
                     " created and used for plotting")
    group.add_option("--ecdf", action="store_true",
                     dest="ecdf", default=False,
                     help="Plot empiracle cumulative density inside each panel"
                     " instead of a normal histogram (turns off log-y)")
    group.add_option("--calc-ranges", action="store_true",
                     dest="calc_ranges", default=False,
                     help="Calculate ranges for distribution plots from"
                     " segmentation data (slower) instead of using whole"
                     " genome data (default).")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Histogram options")
    group.add_option("-n", "--nbins", type="int", metavar="N",
                     dest="nbins", default=NBINS,
                     help="Number of bins for signal distribution"
                     " [default: %default]")
    group.add_option("-c", "--chrom", action="append", metavar="CHROM",
                     dest="chroms", default=None,
                     help="Only perform the analysis on data in CHROM,"
                     " where CHROM is a chromosome name such as chr21 or"
                     " chrX (option can be used multiple times to allow"
                     " multiple chromosomes)")
    parser.add_option_group(group)

    group = OptionGroup(parser, "I/O options")
    group.add_option("--order-tracks", dest="track_order_file",
                     default=None, metavar="FILE",
                     help="If specified, tracks will be displayed"
                     " in the order in FILE. FILE must be a permutation"
                     " of all the printed tracks, one per line, exact"
                     " matches only.")
    group.add_option("--order-labels", dest="label_order_file",
                     default=None, metavar="FILE",
                     help="If specified, label will be displayed"
                     " in the order in FILE. FILE must be a permutation"
                     " of all the labels (after substituting with mnemonics,"
                     " if specified), one per line, exact"
                     " matches only.")
    group.add_option("-i", "--indir", dest="inputdirs",
                     default=None, action="append", metavar="DIR",
                     help="Load data from this directory"
                     " This directory should be the output"
                     " directory of a previous run of this module."
                     " This option can be specified multiple times to"
                     " merge previous results together.")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 2:
        parser.error("Inappropriate number of arguments")

    if options.inputdirs:
        for inputdir in options.inputdirs:
            if inputdir == options.outdir:
                parser.error("Output directory cannot be an input directory")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfilename = args[0]
    genomedatadir = args[1]

    kwargs = dict(options.__dict__)
    outdir = kwargs.pop('outdir')
    validate(bedfilename, genomedatadir, outdir, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""Segtools: tools for exploratory analysis of genomic segmentations

Copyright 2011: Michael Hoffman <mmh1@uw.edu>
Copyright 2009: Orion Buske <stasis@uw.edu>
"""
__version__ = "$Revision: 339 $"

import os
import re
import sys

from collections import defaultdict
from contextlib import contextmanager
from functools import partial
from numpy import array, int64, uint32
from time import time

from pkg_resources import resource_filename

try:
    PKG = __package__  # Python 2.6
except NameError:
    PKG = "segtools"

PKG_R = os.extsep.join([PKG, "R"])

EXT_GZ = "gz"
EXT_PICKLE = "pkl"
EXT_R = "R"

DTYPE_SEGMENT_START = int64
DTYPE_SEGMENT_END = int64
DTYPE_SEGMENT_KEY = uint32
DTYPE_STRAND = '|S1'

class Annotation(object):
    """Base class for representing annotation files (BED/GFF/GTF files)

    chromosomes: a dict
      key: chromosome name
      val: segments, a numpy.ndarray,
           (each row is a (start, end, key, [strand, ...]) struct)
           sorted by start, end
           * These segments are not necessarily non-overlapping
    labels: a dict
      key: int ("label_key")  (a unique id; segment['key'])
      val: printable (str or int)  (what's in the actual file)
           This is the 4th column in a BED file and the 3rd column
           in GFF and GTF files.

    filename: the filename from which the annotation was loaded

    """
    class UnpickleError(Exception):
        pass

    class FilenameError(Exception):
        pass

    class FormatError(Exception):
        pass

    def __init__(self, filename, verbose=True):
        """Returns an Annotation object derived from the given file

        filename: path to a data file in one of the following formats:
          BED3+, GFF, GTF, or pickled Annotation. Format must be specified
          by the extension of the file (bed, gff, gtf, pkl), with all but
          pkl potentially gzipped (gz).
        """

        if not os.path.isfile(filename):
            raise IOError("Could not find file: %s" % filename)

        log("Loading %s from file: %s" %
            (self.__class__.__name__, filename), verbose)
        start = time()
        self._load(filename, verbose=verbose)
        log("Loading finished in %.1f seconds." % (time() - start), verbose)

    @staticmethod
    def _get_file_format(filepath):
        """Determine the file format based upon the file extension"""
        filename = os.path.basename(filepath)
        root, ext = os.path.splitext(filename)

        if ext:
            # If gzip'd, process filename further
            if ext == ".gz":
                root, ext = os.path.splitext(root)

            if ext:
                return ext[1:].lower()  # remove dot

        raise Annotation.FilenameError("File missing appropriate extension:"
                                        " %s" % filepath)

    def _load(self, filename, verbose=True):
        format = self._get_file_format(filename)
        if format == EXT_PICKLE:
            # Read pickled object file
            self._from_pickle(filename, verbose=verbose)
        elif format in set(["bed", "gff", "gtf"]):
            self._from_file(filename, verbose=verbose)
        else:
            raise self.FormatError("Unrecognized extension (%s) on file: %s"
                                   % (format, filename))

    def _iter_rows(self, filename, verbose=True):
        from .bed import read_native as read_bed
        from .gff import read_native as read_gff
        from .common import maybe_gzip_open

        format = self._get_file_format(filename)

        if format == "bed":
            reader = read_bed
        elif format == "gff":
            reader = read_gff
        elif format == "gtf":
            reader = partial(read_gff, gtf=True)
        else:
            raise self.FormatError("Cannot iterate segments in file format:"
                                   " %s" % format)

        log("  Parsing lines from %s format" % format, verbose)

        with maybe_gzip_open(filename) as infile:
            for datum in reader(infile):
                row = {}
                d = datum.__dict__
                if format == "bed":
                    row['chrom'] = d['chrom']
                    row['start'] = d['chromStart']
                    row['end'] = d['chromEnd']
                    row['name'] = d.get('name', "")
                    row['strand'] = d.get('strand', ".")
                elif format == "gff" or format == "gtf":
                    row['chrom'] = d['seqname']
                    row['start'] = d['start']
                    row['end'] = d['end']
                    row['name'] = d.get('feature', "")
                    row['strand'] = d.get('strand', ".")

                if format == "gtf":
                    attrs = datum.attributes
                    row['gene_id'] = attrs['gene_id']
                    row['transcript_id'] = attrs['transcript_id']

                yield row

    def _from_pickle(self, filename, verbose=True):
        import cPickle
        from .common import maybe_gzip_open

        log("  Unpickling %s" % self.__class__.__name__, verbose)

        with maybe_gzip_open(filename, 'rb') as ifp:
            object = cPickle.load(ifp)
            if not issubclass(object.__class__, self.__class__):
                msg = ("Error: Cannot to load an indexed %s object"
                       " as an indexed %s object."
                       % (object.__class__.__name__,
                          self.__class__.__name__))
                raise self.UnpickleError(msg)

            self.__dict__ = object.__dict__

    def _from_file(self, filename, verbose=True):
        """Parses a data file and sets object attributes

        Missing labels will be empty strings ("")

        """
        from .common import inverse_dict

        data = defaultdict(list)  # A dictionary-like object
        label_dict = {}
        last_segment_start = {}  # per chromosome
        unsorted_chroms = set()
        n_label_segments = {}
        n_label_bases = {}
        stranded = None
        for row in self._iter_rows(filename, verbose=verbose):
            chrom = row['chrom']
            start = row['start']
            end = row['end']
            label = row['name']
            strand = row['strand']

            assert end >= start

            # Keep track of sorted chromosomes
            try:
                if start < last_segment_start[chrom]:
                    unsorted_chroms.add(chrom)
            except KeyError:
                pass

            # If any strands are specified, they all should be
            if strand == "+" or strand == "-":
                assert stranded is None or stranded
                stranded = True
            else:
                assert not stranded
                stranded = False

            try:  # Lookup label key
                label_key = label_dict[label]
            except KeyError:  # Map new label to key
                label_key = len(label_dict)
                label_dict[label] = label_key
                n_label_segments[label] = 0
                n_label_bases[label] = 0

            segment = [start, end, label_key]
            if stranded:
                segment.append(strand)

            data[chrom].append(tuple(segment))
            n_label_segments[label] += 1
            n_label_bases[label] += end - start
            last_segment_start[chrom] = start

        # Create reverse dict for field
        labels = inverse_dict(label_dict)

        # convert lists of tuples to array
        dtype = [('start', DTYPE_SEGMENT_START),
                 ('end', DTYPE_SEGMENT_END),
                 ('key', DTYPE_SEGMENT_KEY)]
        if stranded:
            dtype.append(('strand', DTYPE_STRAND))

        chromosomes = dict((chrom, array(segments, dtype=dtype))
                           for chrom, segments in data.iteritems())

        # Sort segments within each chromosome
        if unsorted_chroms:
            log("  Sorting unsorted segments in the following"
                " chromosomes: %s" % ", ".join(unsorted_chroms), verbose)
            for chrom in unsorted_chroms:
                segments = chromosomes[chrom]
                segments.sort(order=['start'])

        self.filename = filename
        self.chromosomes = chromosomes
        self._labels = labels
        self._n_label_segments = n_label_segments
        self._n_label_bases = n_label_bases
        self._inv_labels = label_dict

    def pickle(self, filename=None, verbose=True, clobber=False):
        """Pickle the annotation into an output file"""
        import cPickle
        from .common import check_clobber, maybe_gzip_open

        if filename is None:
            filename = self.filename

        filename = os.extsep.join([filename, EXT_PICKLE, EXT_GZ])

        check_clobber(filename, clobber)
        log("Pickling %s object to file: %s"
            % (self.__class__.__name__, filename), verbose)
        with maybe_gzip_open(filename, 'wb') as ofp:
            cPickle.dump(self, ofp, -1)

    def num_label_segments(self, label):
        return self._n_label_segments[str(label)]

    def num_segments(self):
        return sum(self._n_label_segments.values())

    def num_label_bases(self, label):
        return self._n_label_bases[str(label)]

    def num_bases(self):
        return sum(self._n_label_bases.values())

    def label_key(self, label):
        return self._inv_labels[str(label)]

    def label(self, label_key):
        return self._labels[label_key]

    @property
    def labels(self):
        return dict(self._labels)

    def set_labels(self, labels):
        missing = set(self._labels.keys()).difference(set(labels.keys()))

        if missing:
            raise ValueError("New labels do not cover old label keys: %r"
                             % missing)
        else:
            self._labels = labels

class Segmentation(Annotation):
    """Representation of a segmentation

    Segments must be non-overlapping.
    Strand information is not included.

    Defines additional attributes:
      tracks: a list of track names that were used to obtain the segmentation
      segtool: the tool that was used to obtain this segmentation (e.g. segway)
      * both will be empty if the file was not a BED file or did not contain
        this information
    """


    class SegmentOverlapError(ValueError):
        pass

    def _iter_rows(self, filename, verbose=True):
        """Override default row reading to ignore strand for Segmentations"""
        for row in super(self.__class__,
                         self)._iter_rows(filename, verbose=verbose):
            row['strand'] = '.';
            yield row;

    def _from_file(self, filename, verbose=True):
        """Wrap file loading to ensure no segments overlap

        This is preferred to wrapping __init__ because we don't need to check
        overlapping and don't want to load metadata if the Segmentation is
        being loaded from a pickle file instead of a BED/GFF file.
        """
        super(self.__class__, self)._from_file(filename, verbose=verbose)

        for chrom, segments in self.chromosomes.iteritems():
            # Make sure there are no overlapping segments
            if segments.shape[0] > 1 and \
                    (segments['end'][:-1] > segments['start'][1:]).any():
                raise self.SegmentOverlapError("Found overlapping segments"
                                               " in chromosome: %s" % chrom)

        self.segtool, self.tracks = self.get_bed_metadata(filename)

    @staticmethod
    def get_bed_metadata(filename):
        from .common import maybe_gzip_open

        regexp = re.compile('description="(.*) segmentation of (.*)"')

        segtool = ""
        tracks = []

        with maybe_gzip_open(filename) as ifp:
            line = ifp.readline()

        matching = regexp.search(line)
        if matching:
            segtool = matching.group(1)
            tracks = matching.group(2).split(', ')

        return (segtool, tracks)

# XXX: should be replaced by use of logging module
# will allow percent-substitution only when verbose is on
def log(message, verbose=True, end="\n", file=sys.stderr):
    """Wrapper for logging messages to stderr

    Similar to Python 3.0 print() syntax

    """
    if verbose:
        file.write(str(message))
        file.write(end)

## Die with error message
def die(msg="Unexpected error"):
    log("\nFatal error: %s\n" % msg)
    sys.exit(1)


## Class to handle sourcing, plotting, and calling R functions
class RInterface(object):
    """
    Interface to R environment, providing methods to source files,
    call functions, and plot with R
    """
    _get_filename = partial(resource_filename, PKG_R)
    def __init__(self, files_to_source=[], verbose=True):
        self.started = False
        self._files = tuple(files_to_source)
        self.verbose = verbose
        self._r_console = lambda msg: sys.stderr.write(str(msg))

    def start(self, transcriptfile=None, verbose=None):
        if verbose is not None:
            self.verbose = verbose

        if self.started:
            return

        self.started = True

        # Start up R
        log("Opening connection to R environment.", self.verbose)
        from rpy2.robjects import r, rinterface
        self._r = r
        self._rinterface = rinterface
        self.RError = rinterface.RRuntimeError

        # Set up transcript
        self._transcript = transcriptfile
        if self._transcript:
            print >>self._transcript
            log("Saving transcript to file: %s" % self._transcript.name,
                self.verbose)

        # Source any R files
        for file in self._files:
            self.source(file)

    def source(self, filename):
        """Simplify importing R source in the package"""
        self.start()

        filename_full = self._get_filename(filename)
        if self._transcript:
            print >>self._transcript, "source(%r)" % filename_full

        try:
            self._r.source(filename_full)
        except self.RError:
            die("Failed to load R package: %s\n" % filename_full)

    @classmethod
    def arg_to_text(cls, arg):
        if isinstance(arg, bool):
            return repr(arg).upper() # True -> TRUE, False -> FALSE
        else:
            return repr(arg)

    def call(self, func, *args, **kwargs):
        """Safer way to call R functions (without importing all that junk)

        None values are substituted with empty string
        'ropts' keyword argument is parsed separately and should be None
          or a list of 'param=value' strings to be passed to R.
        """
        self.start()

        from rpy2.robjects import numpy2ri
        # numpy2ri imported for side-effect

        # Make sure there are no None values in args or kwargs
        # rpy2 currently doesn't support passing Nones, so replace them with ""
        for arg_i, arg in enumerate(args):
            if arg is None:
                args[arg_i] = ""


        for key, val in kwargs.iteritems():
            if val is None:
                kwargs[key] = ""

        ropts = kwargs.pop('ropts')
        for opt in ropts:
            if opt:
                key, value = opt.split('=')
                kwargs[key] = value

        # Save call to transcript
        if self._transcript:
            args_text = [self.arg_to_text(arg) for arg in args]
            kwargs_text = ["%s = %s" % (key, self.arg_to_text(value))
                           for key, value in kwargs.iteritems()]
            all_args = ", ".join(args_text + kwargs_text)

            print >>self._transcript, "%s(%s)" % (func, all_args)

        return self._r[func](*args, **kwargs)

    def plot(self, func, *args, **kwargs):
        """Call R function but print timing information

        (kept around for backwards compatibility)
        """
        self.start()
        log("Plotting with R function: %r" % func, self.verbose)
        start = time()

        # Unless verbose, only print R output when there is an error
        r_log = []
        if self.verbose:
            r_console = self._r_console
        else:
            r_console = lambda msg: r_log.append(str(msg))

        self._rinterface.set_writeconsole(r_console)
        try:
            result = self.call(func, *args, **kwargs)
        except self.RError:
            die("Encoundered error within R:\n%s" % "\n  ".join(r_log))

        # Return console back to stderr
        self._rinterface.set_writeconsole(self._r_console)

        log("Plotting completed in %.1f seconds" % (time() - start),
            self.verbose)
        return result


@contextmanager
def open_transcript(dirpath, module, verified=False):
    filename = os.path.join(dirpath, os.extsep.join([module, EXT_R]))
    with open(filename, "w") as res:
        if not verified:
            print >>res, "## Experimental R transcript"
            print >>res, "## You may not be able to run the R code in this file exactly as written."
            print >>res

        yield res

def add_common_options(parser, keys, **kwargs):
    for key in keys:
        if key == 'mnemonic_file':
            parser.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
                              default=None, metavar="FILE",
                              help="If specified, labels will be"
                              " shown using mnemonics found in FILE")
        elif key == 'outdir':
            parser.add_option("-o", "--outdir", metavar="DIR",
                              dest="outdir", default="%(MODULE)s" % kwargs,
                              help="File output directory (will be created"
                              " if it does not exist) [default: %default]")
        elif key == 'clobber':
            parser.add_option("--clobber", action="store_true",
                              dest="clobber", default=False,
                              help="Overwrite any existing output files.")
        elif key == 'quiet':
            parser.add_option("-q", "--quiet", action="store_false",
                              dest="verbose", default=True,
                              help="Do not print diagnostic messages.")
        elif key == 'quick':
            parser.add_option("--quick", action="store_true",
                              dest="quick", default=False,
                              help="Compute values only for one chromosome.")
        elif key == 'replot':
            parser.add_option("--replot", action="store_true",
                              dest="replot", default=False,
                              help="Load data from output tab files and"
                              " regenerate plots instead of recomputing data.")
        elif key == 'noplot':
            parser.add_option("--noplot", action="store_true",
                              dest="noplot", default=False,
                              help="Do not generate any plots.")
        elif key == 'ropts':
            parser.add_option("-R", action="append", metavar="PARAM=VAL",
                              dest="ropts", default=[],
                              help="Pass VAL for PARAM when calling R"
                              " functions. May be specified multiple times.")


if __name__ == "__main__":
    pass




#!/bin/env python

"""
Combine segments from all SEGMENTATION files, labeling with unique labels for
each combination of overlapping that occurs.
Outputs a bed file on
stdout (-o to change), and generates a file in the current directory
that describes the generated labels (use -m to change).
"""

# Author: Orion Buske <stasis@uw.edu>
# Date:   02.02.2010


from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

import os
import sys

from numpy import concatenate

from . import log, Segmentation, die, add_common_options

DEFAULT_HELPFILE = "flat.mnemonics"


class IntervalSegment(object):
    def __init__(self, segment=None, start=None, end=None, keys=[]):
        if segment is None:
            self.start = start
            self.end = end
            self.keys = keys
        else:
            self.start = segment['start']
            self.end = segment['end']
            self.keys = [segment['key']]

    def signature(self):
        return tuple(sorted(set(self.keys)))

    def __str__(self):
        return "[%r, %r, %s]" % (self.start, self.end, self.keys)

    def __repr__(self):
        return "<IntervalSegment %s>" % str(self)

class Interval(object):
    def __init__(self):
        self.start = None
        self.end = None
        self._segments = []  # Not kept sorted

    def add(self, segment):
        """Add a segment to the interval"""
        if not self._segments:
            self.start = segment['start']
            self.end = segment['end']
            self._segments.append(IntervalSegment(segment=segment))
        else:
            segment_start = segment['start']
            segment_end = segment['end']
            segment_key = segment['key']
            assert segment_start < self.end

            # Potentially partition existing segments along boundaries
            segments_to_add = []
            for i, i_segment in enumerate(self._segments):
                i_start = i_segment.start
                i_end = i_segment.end
                i_keys = i_segment.keys
                if i_start < segment_end and \
                        i_end > segment_start:

                    # Maybe add new segment before overlap
                    if i_start < segment_start:
                        new_segment = IntervalSegment(start=i_start,
                                                      end=segment_start,
                                                      keys=i_keys)
                        segments_to_add.append(new_segment)

                    # Change segment to overlap boundary
                    i_segment.start = max(segment_start, i_start)
                    i_segment.end = min(segment_end, i_end)
                    i_segment.keys = i_keys + [segment_key]

                    # Maybe add new segment after overlap
                    if i_end > segment_end:
                        new_segment = IntervalSegment(start=segment_end,
                                                      end=i_end,
                                                      keys=i_keys)
                        segments_to_add.append(new_segment)

            # Maybe add one new segment at the end
            if segment_end > self.end:
                new_segment = IntervalSegment(start=self.end,
                                              end=segment_end,
                                              keys=[segment_key])
                segments_to_add.append(new_segment)
                self.end = segment_end  # Extend interval end

            # Add all new segments to the end of the interval
            self._segments.extend(segments_to_add)


    def flush_to(self, pos):
        """Return segments from the start of the interval to the position"""
        if not self._segments or pos <= self.start:
            return []
        else:
            segments = []
            i = 0
            while i < len(self._segments):
                segment = self._segments[i]
                if segment.end <= pos:
                    # Segment to be completely flushed
                    segments.append(segment)
                    del self._segments[i]
                else:
                    i += 1

            self.start = pos
            return segments

    def flush(self):
        """Flush all remaining segments"""
        self.start = self.end
        segments = self._segments
        self._segments = []
        return segments

    def __repr__(self):
        return "<Interval: %r, %r, %r>" % \
            (self.start, self.end, self._segments)

def join_chrom_segments(chrom, files, segmentations, label_offsets):
    # Combine segments from all segmentations
    segments = None
    for file in files:
        segmentation = segmentations[file]
        label_offset = label_offsets[file]
        try:
            cur_segments = segmentation.chromosomes[chrom]

            # Modify label keys to keep different file segmentations apart
            cur_segments['key'] += label_offset

            # Add current segment array to segments
            if segments is None:
                segments = cur_segments
            else:
                segments = concatenate((segments, cur_segments), axis=0)
        except KeyError:
            pass

    segments.sort()
    return segments

def flatten_segments(segments):
    """Flatten overlapping segments into IntervalSegments"""

    new_segments = []
    if len(segments) == 0:
        return new_segments

    interval = Interval()
    for segment in segments:
        # Process segments up to segment_start
        flushed_segments = interval.flush_to(segment['start'])
        new_segments.extend(flushed_segments)

        # Add new segment to interval
        interval.add(segment)

    # Flush remaining segments from interval
    new_segments.extend(interval.flush())

    return new_segments

def merge_segments(segmentations):
    """Merge together segments from different segmentations

    segmentations: a dict mapping file names to Segmentation objects.
    """

    files = sorted(segmentations.keys())  # Constant file order

    chroms = set()
    label_offsets = {}  # file -> label_offset
    shifted_labels = {}  # shifted_key -> label_string
    for file in files:
        segmentation = segmentations[file]
        chroms.update(segmentation.chromosomes.keys())
        labels = segmentation.labels

        # Shift labels of file to be unique
        label_offset = len(shifted_labels)
        label_offsets[file] = label_offset
        for key, label in labels.iteritems():
            shifted_labels[key + label_offset] = "%s:%s" % (file, label)

    signature_labels = {}  # (shifted_key, ...) -> flat_key
    flat_labels = {}  # flat_key -> label_string
    new_segments = []
    for chrom in chroms:
        segments = join_chrom_segments(chrom, files, segmentations,
                                       label_offsets)
        flat_segments = flatten_segments(segments)

        for flat_segment in flat_segments:
            signature = flat_segment.signature()
            try:
                key = signature_labels[signature]
            except KeyError:
                key = len(signature_labels)
                key_str = "[%s]" % ", ".join([shifted_labels[shifted_key]
                                              for shifted_key in signature])
                signature_labels[signature] = key
                flat_labels[key] = key_str

            new_segment = (chrom, flat_segment.start, flat_segment.end, key)
            new_segments.append(new_segment)

    return flat_labels, new_segments

def filter_segments(labels, segments, filter=None):
    # Do nothing if no filtering
    if filter is None or filter == 0:
        return labels, segments

    if (not isinstance(filter, float) or filter < 0 or filter > 1):
        raise ValueError("Invalid value of filter: %s" % str(filter))

    # Find span of labels and whole segmentation
    print >>sys.stderr, "Calculating span of labels and segmentation...",
    n_seg = 0
    n_keys = dict([(key, 0) for key in labels.iterkeys()])
    for segment in segments:
        key = segment[3]
        length = segment[2] - segment[1]
        n_keys[key] += length
        n_seg += length
    print >>sys.stderr, "done"

    # Find segment labels that need to be filtered
    filtered_labels = dict(labels)  # Copy of original labels
    thresh = n_seg * filter
    for key, n_key in n_keys.iteritems():
        if n_key < thresh:
            del filtered_labels[key]

    # Remove segments not in filtered segment labels
    print >>sys.stderr, "Filtering segment labels below threshold...",
    filtered_segments = []
    for segment in segments:
        key = segment[3]
        if key in filtered_labels:
            filtered_segments.append(segment)
    print >>sys.stderr, "done"

    return filtered_labels, filtered_segments

def print_bed(segments, filename=None):
    """Print segments in BED format to file (or stdout if None)"""
    if filename is None:
        outfile = sys.stdout
    else:
        if os.path.isfile(filename):
            log("Warning: overwriting file: %s" % filename)

        outfile = open(filename, "w")

    for segment in segments:
        outfile.write("%s\n" % "\t".join([str(val) for val in segment]))

    if outfile is not sys.stdout:
        outfile.close()

def print_readme(labels, filename=DEFAULT_HELPFILE):
    if os.path.isfile(filename):
        log("Warning: overwriting file: %s" % filename)

    with open(filename, "w") as ofp:
        ofp.write("%s\n" % "\t".join(["old", "new", "description"]))
        for key, label in labels.iteritems():
            ofp.write("%s\n" % "\t".join([str(key), str(key), label]))

def flatten(files, outfile=None, helpfile=DEFAULT_HELPFILE,
            filter=None, verbose=True):
    segmentations = {}
    for file in files:
        assert os.path.isfile(file)
        nice_filename = os.path.basename(file)
        segmentations[nice_filename] = Segmentation(file, verbose=verbose)

    labels, segments = merge_segments(segmentations)
    labels, segments = filter_segments(labels, segments, filter=filter)
    print_bed(segments, filename=outfile)
    print_readme(labels, filename=helpfile)

def parse_args(args):
    from optparse import OptionParser

    usage = "%prog [OPTIONS] SEGMENTATION..."
    parser = OptionParser(usage=usage, version=__version__,
                          description=__doc__.strip())

    add_common_options(parser, ['quiet'])
    parser.add_option("-m", "--mnemonic-file", dest="helpfile",
                      default=DEFAULT_HELPFILE, metavar="FILE",
                      help="Save mapping information to FILE instead of"
                      " %default (default). This file complies with the"
                      " mnemonic file format.")
    parser.add_option("-o", "--outfile", dest="outfile",
                      default=None, metavar="FILE",
                      help="Save flattened bed file to FILE instead of"
                      " printing to stdout (default)")
    parser.add_option("-f", "--filter", dest="filter",
                      default=None, type="float", metavar="F",
                      help="Don't output new segment labels (and corresponding"
                      " segments) that span less than F*N bases, where"
                      " N is the number of bases covered by the new"
                      " segmentation. This can be used to remove"
                      " extremely uncommon labels (e.g. F = 0.01)"
                      " that are the more likely to be spurious."
                      " Filtering is off by default.")

    options, args = parser.parse_args(args)

    if len(args) == 0:
        parser.error("Inappropriate number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_args(args)

    for arg in args:
        if not os.path.isfile(arg):
            die("Could not find file: %s" % arg)

    kwargs = {"helpfile": options.helpfile,
              "verbose": options.verbose,
              "filter": options.filter,
              "outfile": options.outfile}
    flatten(args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

"""
Evaluates the overlap between two BED files, based upon the spec at:
http://encodewiki.ucsc.edu/EncodeDCC/index.php/Overlap_analysis_tool_specification
"""

# Author: Orion Buske <stasis@uw.edu>
# Date:   August 18, 2009


import os
import sys

from collections import defaultdict
from numpy import bincount, cast, iinfo, invert, logical_or, zeros

from . import Annotation, log, open_transcript, Segmentation, RInterface, \
     add_common_options
from .common import check_clobber, die, get_ordered_labels, make_tabfilename, \
     setup_directory, SUFFIX_GZ, tab_saver
from .html import save_html_div

# A package-unique, descriptive module name used for filenames, etc
MODULE = "overlap"

NAMEBASE = "%s" % MODULE
PERFORMANCE_NAMEBASE = os.extsep.join([NAMEBASE, "performance"])

OVERLAPPING_SEGMENTS_NAMEBASE = os.extsep.join([NAMEBASE, "segments"])
OVERLAPPING_SEGMENTS_FIELDS = ["chrom", "start (zero-indexed)",
                               "end (exclusive)", "group",
                               "[additional fields]"]

HTML_TITLE_BASE = "Overlap statistics"
HTML_TEMPLATE_FILENAME = "overlap_div.tmpl"
SIGNIFICANCE_TEMPLATE_FILENAME = "overlap_significance.tmpl"

NONE_COL = "none"
TOTAL_COL = "total"

MODE_CHOICES = ["segments", "bases"]
MODE_DEFAULT = "bases"
MIDPOINT_CHOICES = ["1", "2", "both"]
SAMPLES_DEFAULT = 5000
REGION_FRACTION_DEFAULT = 0.2
SUBREGION_FRACTION_DEFAULT = 0.2

PNG_SIZE_PER_PANEL = 400  # px
SIGNIFICANCE_PNG_SIZE = 600  # px
HEATMAP_PNG_SIZE = 600 # px

R = RInterface(["common.R", "overlap.R"])

def calc_overlap(subseg, qryseg, quick=False, clobber=False, mode=MODE_DEFAULT,
                 print_segments=False, dirpath=None, verbose=True,
                 min_overlap=1):
    min_overlap = int(min_overlap)

    if print_segments: assert dirpath is not None

    sub_labels = subseg.labels
    qry_labels = qryseg.labels

    # Set up output files if printing overlapping segments
    if print_segments:
        outfiles = {}
        header = "# %s" % "\t".join(OVERLAPPING_SEGMENTS_FIELDS)
        for sub_label_key, sub_label in sub_labels.iteritems():
            outfilename = os.extsep.join([OVERLAPPING_SEGMENTS_NAMEBASE,
                                          sub_label, "txt"])
            outfilepath = os.path.join(dirpath, outfilename)
            check_clobber(outfilepath, clobber=clobber)
            outfiles[sub_label_key] = open(outfilepath, "w")
            outfiles[sub_label_key].write("%s\n" % header)

    counts = zeros((len(sub_labels), len(qry_labels)), dtype="int")
    totals = zeros(len(sub_labels), dtype="int")
    nones = zeros(len(sub_labels), dtype="int")

    for chrom in subseg.chromosomes:
        log("\t%s" % chrom, verbose)

        try:
            qry_segments = qryseg.chromosomes[chrom]
        except KeyError:
            segments = subseg.chromosomes[chrom]
            segment_keys = segments['key']
            # Numpy does not currently support using bincount on unsigned
            # integers, so we need to cast them to signed ints first.
            # To do this safely, we need to make sure the max segment key
            # is below the max signed int value.
            dtype_max = iinfo('int32').max
            assert segment_keys.max() < dtype_max  # Can cast to int32
            segment_keys = cast['int32'](segment_keys)

            # Assumes segment keys are non-negative, consecutive integers
            if mode == "segments":
                key_scores = bincount(segment_keys)

            elif mode == "bases":
                # Weight each segment by its length
                weights = segments['end'] - segments['start']
                key_scores = bincount(segment_keys, weights=weights)
                key_scores.astype("int")
            else:
                raise NotImplementedError("Unknown mode: %r" % mode)

            num_scores = key_scores.shape[0]
            totals[0:num_scores] += key_scores
            nones[0:num_scores] += key_scores
            continue

        # Track upper and lower bounds on range of segments that might
        #   be in the range of the current segment (to keep O(n))
        qry_segment_iter = iter(qry_segments)
        qry_segment = qry_segment_iter.next()
        qry_segments_in_range = []
        for sub_segment in subseg.chromosomes[chrom]:
            substart = sub_segment['start']
            subend = sub_segment['end']
            sublabelkey = sub_segment['key']
            sublength = subend - substart
            # Compute min-overlap in terms of bases, if conversion required
            min_overlap_bp = min_overlap

            if mode == "segments":
                full_score = 1
            elif mode == "bases":
                full_score = sublength

            # Add subject segment to the total count
            totals[sublabelkey] += full_score

            # Remove from list any qry_segments that are now too low
            i = 0
            while i < len(qry_segments_in_range):
                segment = qry_segments_in_range[i]
                if segment['end'] - min_overlap_bp < substart:
                    del qry_segments_in_range[i]
                else:
                    i += 1

            # Advance qry_segment pointer to past sub_segment, updating list
            while qry_segment is not None and \
                    qry_segment['start'] <= subend - min_overlap_bp:
                if qry_segment['end'] - min_overlap_bp >= substart:
                    # qry_segment overlaps with sub_segment
                    qry_segments_in_range.append(qry_segment)
                try:
                    qry_segment = qry_segment_iter.next()
                except StopIteration:
                    qry_segment = None

            # Skip processing if there aren't any segments in range
            if len(qry_segments_in_range) == 0:
                nones[sublabelkey] += full_score
                continue

            # Scan list for subset that actually overlap current segment
            overlapping_segments = []
            for segment in qry_segments_in_range:
                if segment['start'] <= subend - min_overlap_bp:
                    assert segment['end'] - min_overlap_bp >= substart
                    overlapping_segments.append(segment)

            # Skip processing if there are no overlapping segments
            if len(overlapping_segments) == 0:
                nones[sublabelkey] += full_score
                continue

            if print_segments:
                for segment in overlapping_segments:
                    values = [chrom,
                              segment['start'],
                              segment['end'],
                              qry_labels[segment['key']]]
                    # Add a source if there is one
                    try:
                        values.append(qryseg.sources[segment['source_key']])
                    except (AttributeError, IndexError):
                        pass
                    # Add any other data in the segment
                    try:
                        values.extend(tuple(segment)[4:])
                    except IndexError:
                        pass
                    values = [str(val) for val in values]
                    outfiles[sublabelkey].write("%s\n" % "\t".join(values))

            # Organize overlapping_segments by qrylabelkey
            label_overlaps = defaultdict(list)  # Per qrylabelkey
            for segment in overlapping_segments:
                label_overlaps[segment['key']].append(segment)
            label_overlaps = dict(label_overlaps)  # Remove defaultdict

            if mode == "segments":
                # Add 1 to count for each group that overlaps at least
                # one segment
                for qrylabelkey in label_overlaps:
                    counts[sublabelkey, qrylabelkey] += 1
            elif mode == "bases":
                # Keep track of total covered by any labels
                covered = zeros(sublength, dtype="bool")
                for qrylabelkey, segments in label_overlaps.iteritems():
                    # Look at total covered by single label
                    label_covered = zeros(sublength, dtype="bool")
                    for segment in segments:
                        qrystart = segment['start']
                        qryend = segment['end']
                        qrylabelkey = segment['key']
                        # Define bounds of coverage
                        cov_start = max(qrystart - substart, 0)
                        cov_end = min(qryend - substart, sublength)
                        label_covered[cov_start:cov_end] = True

                    # Add the number of bases covered by this segment
                    counts[sublabelkey, qrylabelkey] += label_covered.sum()
                    covered = logical_or(covered, label_covered)

                # See how many bases were never covered by any segment
                nones[sublabelkey] += invert(covered).sum()

        if quick: break

    if print_segments:
        for outfile in outfiles.itervalues():
            outfile.close()

    return (counts, totals, nones)

def make_tab_row(col_indices, data, none, total):
    row = [data[col_i] for col_i in col_indices]
    row.extend([none, total])
    return row

## Saves the data to a tab file
def save_tab(dirpath, row_labels, col_labels, counts, totals, nones, mode,
             namebase=NAMEBASE, clobber=False, verbose=True):
    assert counts is not None and totals is not None and nones is not None

    row_label_keys, row_labels = get_ordered_labels(row_labels)
    col_label_keys, col_labels = get_ordered_labels(col_labels)
    colnames = [col_labels[label_key] for label_key in col_label_keys]
    metadata = {"mode": mode}
    with tab_saver(dirpath, namebase, clobber=clobber, verbose=verbose,
                   metadata=metadata) as count_saver:
        header = [""] + colnames + [NONE_COL, TOTAL_COL]
        count_saver.writerow(header)
        for row_label_key in row_label_keys:
            row = make_tab_row(col_label_keys, counts[row_label_key],
                               nones[row_label_key], totals[row_label_key])
            row.insert(0, row_labels[row_label_key])
            count_saver.writerow(row)

def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
              row_mnemonic_file=None, col_mnemonic_file=None, ropts=None,
              cluster=False, max_contrast=False, transcriptfile=None):
    R.start(transcriptfile, verbose=verbose)

    tabfilename = make_tabfilename(dirpath, namebase)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.overlap.heatmap", dirpath, namebase, tabfilename,
           mnemonic_file=row_mnemonic_file,
           col_mnemonic_file=col_mnemonic_file,
           clobber=clobber, cluster=cluster,
           max_contrast=max_contrast, ropts=ropts)

def save_performance_plot(dirpath, namebase=PERFORMANCE_NAMEBASE,
                          clobber=False, verbose=True,
                          row_mnemonic_file=None, col_mnemonic_file=None,
                          transcriptfile=None, ropts=None):
    R.start(transcriptfile, verbose=verbose)

    tabfilename = make_tabfilename(dirpath, NAMEBASE)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.overlap.performance", dirpath, namebase, tabfilename,
           mnemonic_file=row_mnemonic_file,
           col_mnemonic_file=col_mnemonic_file,
           clobber=clobber, ropts=ropts)


def save_html(dirpath, bedfilename, featurefilename, mode,
              mnemonicfile=None, clobber=False, verbose=True):
    bedfilename = os.path.basename(bedfilename)
    featurebasename = os.path.basename(featurefilename)
    extra_namebases = {"performance": PERFORMANCE_NAMEBASE}

    title = "%s (%s)" % (HTML_TITLE_BASE, featurebasename)

    significance = ""
    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  title=title, tables={"":(PERFORMANCE_NAMEBASE, "exact")},
                  mnemonicfile=mnemonicfile, verbose=verbose,
                  extra_namebases=extra_namebases,
                  module=MODULE, by=mode, significance=significance,
                  bedfilename=bedfilename, featurefilename=featurebasename)

def is_file_type(filename, ext):
    """Return True if the filename is of the given extension type (e.g. 'txt')

    Allows g-zipping

    """
    base = os.path.basename(filename)
    if base.endswith(SUFFIX_GZ):
        base = base[:-3]
    return base.endswith("." + ext)

## Package entry point
def overlap(bedfilename, featurefilename, dirpath, regionfilename=None,
            clobber=False, quick=False, print_segments=False,
            mode=MODE_DEFAULT, samples=SAMPLES_DEFAULT,
            region_fraction=REGION_FRACTION_DEFAULT,
            subregion_fraction=SUBREGION_FRACTION_DEFAULT, min_overlap=1,
            mnemonic_file=None, feature_mnemonic_file=None,
            replot=False, noplot=False, cluster=False, max_contrast=False,
            verbose=True, ropts=None):
    if not replot:
        setup_directory(dirpath)

        segmentation = Segmentation(bedfilename, verbose=verbose)
        features = Annotation(featurefilename, verbose=verbose)

        seg_labels = segmentation.labels
        feature_labels = features.labels

        # Overlap of segmentation with features
        log("Measuring overlap:", verbose)

        counts, nones, totals = \
            calc_overlap(segmentation, features, clobber=clobber,
                         mode=mode, min_overlap=min_overlap,
                         print_segments=print_segments,
                         quick=quick, dirpath=dirpath, verbose=verbose)

        save_tab(dirpath, seg_labels, feature_labels,
                 counts, nones, totals, mode=mode,
                 clobber=clobber, verbose=verbose)

    if not noplot:
        with open_transcript(dirpath, MODULE, True) as transcriptfile:
            # Performance plot is only valid for bases
            if mode == "bases":
                save_performance_plot(\
                    dirpath, clobber=clobber, verbose=verbose,
                    row_mnemonic_file=mnemonic_file, ropts=ropts,
                    col_mnemonic_file=feature_mnemonic_file,
                    transcriptfile=transcriptfile)
            else:
                log("Not in base-mode, so skipping performance plot", verbose)

            save_plot(dirpath, clobber=clobber, cluster=cluster,
                      verbose=verbose, row_mnemonic_file=mnemonic_file,
                      col_mnemonic_file=feature_mnemonic_file, ropts=ropts,
                      max_contrast=max_contrast, transcriptfile=transcriptfile)

    save_html(dirpath, bedfilename, featurefilename, mode=mode,
              mnemonicfile=mnemonic_file, clobber=clobber, verbose=verbose)

def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATION"
    description = "SEGMENTATION and ANNOTATION files should be in BED, GFF, \
or GTF format (grouped on 'name'/'feature' columns). \
Results summarize the overlap \
of SEGMENTATION groups with ANNOTATION groups. The symmetric analysis can \
be performed by rerunning the program with the input file arguments swapped \
(and a different output directory). A rough specification can be found here: \
http://encodewiki.ucsc.edu/EncodeDCC/index.php/\
Overlap_analysis_tool_specification"

    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version,
                          description=description)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'quick', 'replot',
                               'noplot'])
    group.add_option("--cluster", action="store_true",
                     dest="cluster", default=False,
                     help="Cluster rows and columns in heat map plot")
    group.add_option("-p", "--print-segments", action="store_true",
                     dest="print_segments", default=False,
                     help=("For each group"
                     " in the SEGMENTATION, a separate output file will be"
                     " created that contains a list of all the segments that"
                     " the group was found to overlap with. Output files"
                     " are named %s.X.txt, where X is the name"
                     " of the SEGMENTATION group.")
                     % OVERLAPPING_SEGMENTS_NAMEBASE)
    group.add_option("--max-contrast", action="store_true",
                     dest="max_contrast", default=False,
                     help="Saturate color range instead of having it go from"
                     " 0 to 1")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Parameters")
    group.add_option("-b", "--by", choices=MODE_CHOICES, metavar="MODE",
                     dest="mode", type="choice", default=MODE_DEFAULT,
                     help="One of: "+str(MODE_CHOICES)+", which determines the"
                     " definition of overlap. @segments: The value"
                     " associated with two features overlapping will be 1 if"
                     " they overlap, and 0 otherwise. @bases: The value"
                     " associated with two features overlapping will be"
                     " number of base pairs which they overlap."
                     " [default: %default]")
    group.add_option("--min-overlap", type="int",
                     dest="min_overlap", default=1, metavar="N",
                     help="The minimum number of base pairs that two"
                     " features must overlap for them to be classified as"
                     " overlapping. This integer can be either positive"
                     " (features overlap only if they share at least this"
                     " many bases) or negative (features overlap if there"
                     " are no more than this many bases between them). Both"
                     " a negative min-overlap and --by=bases cannot be"
                     " specified together. [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Files")
    add_common_options(group, ['mnemonic_file'])
    group.add_option("--feature-mnemonic-file", metavar="FILE",
                     dest="feature_mnemonic_file", default=None,
                     help="If specified, ANNOTATION groups will be shown"
                     " using mnemonics found in FILE.")
    add_common_options(group, ['outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) < 2:
        parser.error("Insufficient number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    kwargs = dict(options.__dict__)
    args.append(kwargs.pop('outdir'))
    overlap(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/bin/env python

"""
Generates a new segmentation by first relabeling the segments in a given
SEGMENTATION according to the label mappings in MNEMONICFILE and then
merging overlapping segments with the same final label.
Outputs the new segmentation in bed format to stdout (-o to change).
"""

# Author: Orion Buske <stasis@uw.edu>
# Date:   05.16.2011

from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

import os
import sys

from . import log, Segmentation, die, add_common_options
from .common import map_mnemonics, get_ordered_labels


def print_bed(segments, filename=None):
    """Print segments in BED format to file (or stdout if None)"""
    if filename is None:
        outfile = sys.stdout
    else:
        if os.path.isfile(filename):
            log("Warning: overwriting file: %s" % filename)

        outfile = open(filename, "w")

    for segment in segments:
        outfile.write("%s\n" % "\t".join([str(val) for val in segment]))

    if outfile is not sys.stdout:
        outfile.close()

def relabel(segfile, mnemonicfile, outfile=None, verbose=True):
    assert os.path.isfile(segfile)
    segmentation = Segmentation(segfile, verbose=verbose)

    labels = segmentation.labels
    mnemonics = map_mnemonics(labels, mnemonicfile)
    ordered_labels, mnemonics = get_ordered_labels(labels, mnemonics)

    print >>sys.stderr, "Found labels:\n%s" % labels

    print >>sys.stderr, "Found mnemonics:\n%s" % mnemonics

    if outfile is None:
        out = sys.stdout
    else:
        out = open(outfile, "w")


    for chrom, segments in segmentation.chromosomes.iteritems():
        def print_segment(segment, label):
            tokens = [chrom, segment['start'], segment['end'],
                      mnemonics[segment['key']]]
            try:
                tokens.extend([0, segment['strand']])
            except IndexError:
                pass
            out.write('\t'.join([str(token) for token in tokens]))
            out.write('\n')

        prev_segment = segments[0]
        prev_label = mnemonics[prev_segment['key']]
        for segment in segments[1:]:
            # Merge adjacent segments of same label
            label = mnemonics[segment['key']]
            if label == prev_label and \
                    segment['start'] == prev_segment['end']:
                try:
                    strands_match = (segment['strand'] == prev_segment['strand'])
                except IndexError:
                    strands_match = True

                if strands_match:
                    prev_segment['end'] = segment['end']
                    continue

            print_segment(prev_segment, prev_label)
            prev_segment, prev_label = segment, label
        print_segment(prev_segment, prev_label)

    if out is not sys.stdout:
        out.close()

def parse_args(args):
    from optparse import OptionParser

    usage = "%prog [OPTIONS] SEGMENTATION MNEMONICFILE"
    parser = OptionParser(usage=usage, version=__version__,
                          description=__doc__.strip())
    add_common_options(parser, ['quiet'])
    parser.add_option("-o", "--outfile", dest="outfile",
                      default=None, metavar="FILE",
                      help="Save relabeled bed file to FILE instead of"
                      " printing to stdout (default)")

    options, args = parser.parse_args(args)

    if len(args) != 2:
        parser.error("Inappropriate number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_args(args)

    for arg in args:
        if not os.path.isfile(arg):
            die("Could not find file: %s" % arg)

    kwargs = dict(options.__dict__)
    relabel(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

"""
html.py

HTML utilities for segtools
"""

import os
import re
import sys
import time

from functools import partial
from pkg_resources import resource_string
from shutil import copy
from string import Template

from . import log, Segmentation, die, add_common_options
from .common import check_clobber, get_ordered_labels, make_divfilename, \
     make_id, make_filename, make_tabfilename, map_mnemonics, NICE_EXTS, \
     PKG_RESOURCE

MNEMONIC_TEMPLATE_FILENAME = "mnemonic_div.tmpl"
HEADER_TEMPLATE_FILENAME = "html_header.tmpl"
FOOTER_TEMPLATE_FILENAME = "html_footer.tmpl"
GENOMEBROWSER_URL = "http://genome.ucsc.edu/cgi-bin/hgTracks?org=human&hgt.customText=track"
GENOMEBROWSER_OPTIONS = {"autoScale":"off",
                         "viewLimits":"0:1",
                         "visibility":"full",
                         "itemRgb":"on",
                         "name":"segtools"}
GENOMEBROWSER_LINK_TMPL = """
<li>Link to view this segmentation in the UCSC genome browser:<br />
<script type="text/javascript">print_genomebrowser_link("%s");</script>
</li>"""

DESCRIPTION_MODULE = ("description", "Segmentation information")
MNEMONIC_MODULE = ("mnemonics", "Mnemonics")

template_string = partial(resource_string, PKG_RESOURCE)

def template_substitute(filename):
    """
    Simplify import resource strings in the package
    """
    return Template(template_string(filename)).safe_substitute

def tuple2link(entry):
    """entry should be a (url, text) tuple"""
    return '<a href="%s">%s</a>' % entry

def list2html(list, code=False, link=False):
    """
    If link is True: each element of the list should be a (divID, label) tuple
    """
    result = ["<ul>"]
    entrystr = "%s"
    if link:
        entrystr = '<a href="#%s">%s</a>'
    if code:
        entrystr = "<code>%s</code>"

    entrystr = "<li>%s</li>" % entrystr
    for entry in list:
        result.append(entrystr % entry)

    result.append("</ul>")
    return "\n".join(result)

def tab2html(tabfile, header=True, mnemonicfile=None):
    """
    Given a tab file table with a header row, generates an html string which
    for pretty-display of the table data.
    """
    if not os.path.isfile(tabfile):
        return "<File not found>"

    result = []
    result.append('\n<table border="1" cellpadding="4" cellspacing="1">')
    # if the tabfile exists, write in htmlhandle an html table
    with open(tabfile) as ifp:
        # Read past comments
        line = ifp.readline()
        while line.startswith("#"):
            line = ifp.readline()

        if header:
            # Write colname row from header row of file
            fields = line.split("\t")
            result.append("<tr>")

            for f in fields:
                result.append('<td style="background-color:'
                             'rgb(204, 204, 204)">%s</td>' % f)

            result.append("</tr>")
            line = ifp.readline()

        lines = [line] + ifp.readlines()
        rows = [line.split("\t") for line in lines]
        row_names = [row[0] for row in rows]
        # Make basic labels for row names
        row_order = range(0, len(rows))
        row_labels = dict(zip(row_order, row_names))
        if mnemonicfile:
            # Substitute these labels with mnemonics
            mnemonics = map_mnemonics(row_labels, mnemonicfile)
            row_order, row_labels = get_ordered_labels(row_labels,
                                                       mnemonics)

        for row_key in row_order:
            entry = ["<tr>"]
            fields = rows[row_key]
            fields[0] = row_labels[row_key]  # Replace row_name
            for f in fields:
                entry.append("<td>%s</td>" % f)

            entry.append("</tr>")
            result.append("".join(entry))

    result.append("</table>\n")
    return "\n".join(result)

def find_output_files(dirpath, namebase, d={}, tag=""):
    exts = NICE_EXTS
    # Add filenames of common present files to dict
    for extname, ext in exts.iteritems():
        filename = make_filename(dirpath, namebase, ext)
        if os.path.isfile(filename):
            key = "%s%sfilename" % (tag, extname)
            assert key not in d
            d[key] = filename

    return d

def form_template_dict(dirpath, namebase, module=None, extra_namebases={},
                       mnemonicfile=None, tables={}, **kwargs):
    """
    Given information about the current validation, generates a dictionary
    suitable for HTML template substitution.

    The output directory (dirpath) is searched for files of the form:
    <namebase>.<ext> for common exts. If found, the filename is linked
    in under the variable <ext>filename

    extra_namebases: a dict of tag -> namebase string
    For each extra namebase, any found files will be linked under
    <tag><ext>filename, as opposed to the main namebase, which is just
    under <ext>filename.

    module: The name of the module generating the dict. If specified,
    an id variable will be generated based upon the module and dirpath
    (a pseudo-unique identifier for the div file).

    tables: a dict from tag -> to table file namebase string.
    The rownames of the tables will try to be substituted with the mnemonics
    in mnemonicfile, if provided. Variable of the form <tag>table and
    <tag>tablefilename will be created with the table HTML and table file
    name, respectively. The dict value can also be a tuple of two elements,
    where the first is the namebase string and the second is a string
    mode. Currently, only None and "exact" are supported,
    with the latter causing the table to not be mnemonic-substituted.

    Any other keyword args supplied are linked into the dictionary.
    """
    # Find default files for all namebases
    d = {}
    find_output_files(dirpath, namebase, d=d)
    for tag, nb in extra_namebases.iteritems():
        find_output_files(dirpath, nb, d=d, tag=tag)

    if module is not None:
        arg = "id"
        assert arg not in d
        d[arg] = make_id(module, dirpath)

    # Add any tables
    for tag, table in tables.iteritems():
        if isinstance(table, tuple):
            table, tablemode = table
        else:
            tablemode = None

        tablefilename = make_tabfilename(dirpath, table)
        filearg = "%stablefilename" % tag
        tablearg = "%stable" % tag
        if tablemode is None:
            val = tab2html(tablefilename, mnemonicfile=mnemonicfile)
        elif tablemode == "exact":
            val = tab2html(tablefilename)
        else:
            raise ValueError("Invalid table mode: %s" % tablemode)

        assert filearg not in d and tablearg not in d  # Don't overwrite
        d[filearg] = tablefilename
        d[tablearg] = val

    for arg, val in kwargs.iteritems():
        assert arg not in d  # Don't overwrite
        d[arg] = val

    return d

def write_html_div(dirpath, namebase, html, clobber=False):
    """
    Write the given html div string to an appropriate file
    """
    filename = make_divfilename(dirpath, namebase)
    check_clobber(filename, clobber)

    with open(filename, "w") as ofp:
        ofp.write("%s\n" % html)

def save_html_div(template_filename, dirpath, namebase,
                  clobber=False, verbose=True, **kwargs):
    """Save an HTML div file for a module run by subsituting a template file"""
    fields = form_template_dict(dirpath, namebase, **kwargs)
    div_filename = make_divfilename(dirpath, namebase)
    log("Saving html data to file: %s" % div_filename, verbose)
    try:
        html = template_substitute(template_filename)(fields)
    except KeyError, e:
        log("Error: Missing data: %s. Skipping HTML output." % e)
        return

    write_html_div(dirpath, namebase, html, clobber=clobber)

def form_mnemonic_div(mnemonicfile, results_dir, clobber=False, verbose=True):
    """Copy mnemonic file to results_dir and create the HTML div with a link"""
    filebase = os.path.basename(mnemonicfile)
    link_file = os.path.join(results_dir, filebase)

    if os.path.exists(link_file) and os.path.samefile(mnemonicfile, link_file):
        log("Mnemonic file already in place: %s" % mnemonicfile)
    else:
        check_clobber(link_file, clobber)

        try:
            copy(mnemonicfile, link_file)
        except (IOError, os.error):
            log("Error: could not copy %s to %s. Linking"
                " the the former." % (mnemonicfile, link_file))
            link_file = mnemonicfile  # Link to the existing file
        else:
            log("Copied %s to %s" % (mnemonicfile, link_file), verbose)

    fields = {}
    fields["tabfilename"] = link_file
    fields["table"] = tab2html(mnemonicfile)
    div = template_substitute(MNEMONIC_TEMPLATE_FILENAME)(fields)
    return div

def make_genomebrowser_url(options, urltype):
    """Makes URL for genomebrowser (minus javascript-added file path)

    urltype: either "data" or "bigData"

    """
    url = GENOMEBROWSER_URL
    for k, v in options.iteritems():
        url += " %s=%s " % (k, v)
    url += " %sUrl=" % urltype
    return url

def form_html_header(bedfilename, modules, layeredbed=None, bigbed=None):
    segtool, segtracks = Segmentation.get_bed_metadata(bedfilename)
    bedfilebase = os.path.basename(bedfilename)
    fields = {}
    fields["bedfilename"] = bedfilebase
    fields["numsegtracks"] = len(segtracks)
    fields["segtracks"] = list2html(segtracks, code=True)
    fields["segtool"] = segtool
    fields["bedmtime"] = time.strftime(
        "%m/%d/%Y %I:%M:%S %p", time.localtime(os.path.getmtime(bedfilename)))
    fields["modules"] = list2html(modules, link=True)
    fields["otherbeds"] = ""
    fields["genomebrowserurl"] = ""
    fields["genomebrowserlink"] = ""

    otherbeds = []
    if layeredbed:
        try:
            layeredbed = os.path.relpath(layeredbed)
        except AttributeError:
            pass
        otherbeds.append(tuple2link((layeredbed, "layered")))
    if bigbed:
        try:
            bigbed = os.path.relpath(bigbed)
        except AttributeError:
            pass
        otherbeds.append(tuple2link((bigbed, "bigBed")))

    if layeredbed or bigbed:
        options = GENOMEBROWSER_OPTIONS
        options["description"] = bedfilebase
        # Specify type (only) if using bigBed
        if bigbed:
            options["type"] = "bigBed"
            urltype = "bigData"
            datafile = bigbed
        else:
            urltype = "data"
            datafile = layeredbed
            if "type" in options:
                del options["type"]

        # Specify genomebrowser values to substitute
        fields["genomebrowserurl"] = make_genomebrowser_url(options, urltype)
        fields["genomebrowserlink"] = GENOMEBROWSER_LINK_TMPL % datafile

    if len(otherbeds) > 0:
        fields["otherbeds"] = "(%s)" % (", ".join(otherbeds))

    header = template_substitute(HEADER_TEMPLATE_FILENAME)(fields)
    return header

def form_html_footer():
    return template_substitute(FOOTER_TEMPLATE_FILENAME)()

def find_divs(rootdir=os.getcwd(), verbose=True):
    """Look one level deep in directory, adding any .div files found.
    """
    divs = []
    log("Searching directories under %s for files ending in .div..." % \
        rootdir)
    for foldername in sorted(os.listdir(rootdir)):
        folderpath = os.path.join(rootdir, foldername)
        if os.path.isdir(folderpath):
            for filename in sorted(os.listdir(folderpath)):
                root, ext = os.path.splitext(filename)
                if ext == ".div":
                    filepath = os.path.join(folderpath, filename)
                    divs.append(filepath)
                    log("Found div: %s" % filename, verbose)

    if not divs:
        log("Found no files matching %s/*/*.div" % rootdir)

    return divs

def make_html_report(bedfilename, results_dir, outfile, mnemonicfile=None,
                     clobber=False, verbose=True,
                     layeredbed=None, bigbed=None):
    check_clobber(outfile, clobber)

    divs = find_divs(results_dir, verbose=verbose)
    if len(divs) == 0:
        die("Make sure to run this from the parent directory of the"
            " module output directories or specify the --results-dir option")

    body = []
    modules = [DESCRIPTION_MODULE]
    if mnemonicfile is not None:
        modules.append(MNEMONIC_MODULE)
        div = form_mnemonic_div(mnemonicfile, results_dir, clobber=clobber,
                                verbose=verbose)
        body.append(div)

    regex = re.compile('"module" id="(.*?)".*?<h.>.*?</a>\s*(.*?)\s*</h.>',
                       re.DOTALL)
    for div in divs:
        with open(div) as ifp:
            divstring = "".join(ifp.readlines())
            matching = regex.search(divstring)
            assert matching
            module = (matching.group(1), matching.group(2))
            modules.append(module)
            body.append(divstring)

    header = form_html_header(bedfilename, modules,
                              layeredbed=layeredbed,
                              bigbed=bigbed)
    footer = form_html_footer()

    components = [header] + body + [footer]
    separator = "<br /><hr>"
    with open(outfile, "w") as ofp:
        ofp.write("%s\n" % separator.join(components))

def parse_args(args):
    from optparse import OptionGroup, OptionParser
    usage = "%prog [OPTIONS] SEGMENTATION"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet'])
    parser.add_option_group(group)

    group = OptionGroup(parser, "Linking")
    group.add_option("-m", "--mnemonic-file", dest="mnemonicfile",
                     default=None, metavar="FILE",
                     help="If specified, this mnemonic mapping will be"
                     " included in the report (this should be the same"
                     " mnemonic file used by the individual modules).")
    group.add_option("-L", "--layered-bed", dest="layeredbed",
                     default=None, metavar="FILE",
                     help="If specified, this layered BED file will be linked"
                     " into the the HTML document (assumed to be the same"
                     " data as in SEGMENTATION)")
    group.add_option("-B", "--big-bed", dest="bigbed",
                     default=None, metavar="FILE",
                     help="If specified, this bigBed file will be linked into"
                     " the the HTML document and a UCSC genome brower link"
                     " will be generated for it (assumed to be the same data"
                     " as in SEGMENTATION)")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output")
    group.add_option("--results-dir", dest="resultsdir",
                     default=".", metavar="DIR",
                     help="This should be the directory containing all the"
                     " module output directories (`ls` should return things"
                     " like \"length_distribution/\", etc)"
                     " [default: %default]")
    group.add_option("-o", "--outfile", metavar="FILE",
                     dest="outfile", default="index.html",
                     help="HTML report file (must be in current directory"
                     " or the links will break [default: %default]")
    parser.add_option_group(group)

    options, args = parser.parse_args(args)

    if options.outfile != os.path.basename(options.outfile):
        parser.error("Output file must be in current directory"
                     " (otherwise the reource paths get all messed up)")

    if len(args) < 1:
        parser.error("Insufficient number of arguments")

    return options, args

def main(args=sys.argv[1:]):
    options, args = parse_args(args)
    bedfilename = args[0]
    kwargs = dict(options.__dict__)

    outfile = kwargs.pop('outfile')
    results_dir = kwargs.pop('resultsdir')
    make_html_report(bedfilename, results_dir, outfile, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

"""
Provides command-line and package entry points for analyzing the observed
segmentation label transitions in the given BED-formatted segmentation.


Accepts an input file containing a matrix of transition
probabilities and generates a heatmap of the matrix
in pdf and png formats.

Optionally generates a heatmap of the ratio of reciprocal
transition probabilities (a->b/b->a)

Optionally generates a graphviz image of a thresholded
form of the transition matrix

XXX: Graphiz currently assumes integer labels in sorted order in file

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE = "transition"


import os
import sys


from numpy import empty, loadtxt, where, zeros
from subprocess import call

from . import log, Segmentation, die, RInterface, open_transcript, \
     add_common_options
from .common import check_clobber, get_ordered_labels, make_dotfilename, \
     make_pdffilename, make_pngfilename, make_namebase_summary, \
     make_tabfilename, map_mnemonics, setup_directory, tab_saver
from .html import save_html_div
from .mnemonics import create_mnemonic_file

NAMEBASE = "%s" % MODULE
NAMEBASE_SUMMARY = make_namebase_summary(NAMEBASE)
NAMEBASE_GRAPH = os.extsep.join([NAMEBASE, "graph"])

HTML_TEMPLATE_FILENAME = "transition_div.tmpl"
HTML_TITLE = "Segment label transitions"

P_THRESH = 0.15  # Default
Q_THRESH = 0.0

BASE_WEIGHT = 0.3  # Base edge/arrow weight
EDGE_WEIGHT = 2.7
ARROW_WEIGHT = 1.7

R = RInterface(["common.R", "transition.R"])

## Returns array of row-normalized probabilities corresponding
## to transition probabilities between labels
## Start labels are rows, end labels are cols
def calc_transitions(segmentation):
    labels = segmentation.labels
    self_transition_count = 0
    # NxN matrix for transition counts
    counts = zeros((len(labels), len(labels)), dtype="int")
    for segments in segmentation.chromosomes.itervalues():
        # Inch along rows, looking at all pairs
        row_it = iter(segments)
        row1 = None
        for row2 in row_it:
            if row1 is not None:  # Skip first loop
                # Increment transition only if directly adjacent
                if row1['end'] == row2['start']:
                    label_key1 = row1['key']
                    label_key2 = row2['key']
                    counts[label_key1, label_key2] += 1
                    if label_key1 == label_key2:
                        if self_transition_count == 0:
                            log("WARNING: unexpected self-transition"
                                "%s -> %s\n\tRow 1: %s\n\tRow 2: %s" % \
                                    (labels[label_key1], labels[label_key2],
                                     row1, row2))
                        self_transition_count += 1

            row1 = row2  # Inch forward

    if self_transition_count > 0:
        log("WARNING: %d self-transitions observed" % self_transition_count)

    # Row-normalize to turn counts into probabilities
    probs = empty(counts.shape, dtype='double')
    for i, row in enumerate(counts):
        probs[i] = row / sum(row)

    return labels, probs

def load_transition_probs(dirpath, namebase=NAMEBASE):
    filename = make_tabfilename(dirpath, namebase)
    return loadtxt(filename)

def save_tab(labels, probs, dirpath, clobber=False, verbose=True):
    ordered_keys, labels = get_ordered_labels(labels)

    # Get fieldnames in order
    fieldnames = list(labels[key] for key in ordered_keys)
    with tab_saver(dirpath, NAMEBASE, fieldnames, verbose=verbose,
                   clobber=clobber, header=False) as saver:
        for start_key in ordered_keys:
            prob_row = probs[start_key]
            row = {}
            for end_key in ordered_keys:
                row[labels[end_key]] = "%.5f" % prob_row[end_key]
            saver.writerow(row)

def save_html(dirpath, p_thresh, q_thresh, clobber=False, verbose=True):
    extra_namebases = {"graph": NAMEBASE_GRAPH}

    if p_thresh > 0:
        thresh = "P > %s" % p_thresh
    elif q_thresh > 0:
        thresh = "P above %.2th quantile" % q_thresh

    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, extra_namebases=extra_namebases,
                  title=HTML_TITLE, thresh=thresh, verbose=verbose)

def save_plot(dirpath, namebase=NAMEBASE, filename=None, ddgram=False,
              clobber=False, mnemonic_file=None, verbose=False, gmtk=False,
              transcriptfile=None, ropts=None):
    """
    if filename is specified, it is used instead of dirpath/namebase.tab
    """
    R.start(verbose=verbose, transcriptfile=transcriptfile)

    if filename is None:
        filename = make_tabfilename(dirpath, namebase)

    if not os.path.isfile(filename):
        die("Unable to find tab file: %s" % filename)

    R.plot("save.transition", dirpath, namebase, filename, clobber=clobber,
           mnemonic_file=mnemonic_file, ddgram=ddgram,
           gmtk=gmtk, ropts=ropts)

def save_graph(labels, probs, dirpath, q_thresh=Q_THRESH, p_thresh=P_THRESH,
               clobber=False, mnemonic_file=None, fontname="Helvetica",
               lenient_thresh=True, gene_graph=False, namebase=NAMEBASE_GRAPH,
               verbose=True):
    assert labels is not None and probs is not None
    try:
        import pygraphviz as pgv
    except ImportError:
        log("Unable to load PyGraphviz library. Skipping transition graph")
        return

    dotfilename = make_dotfilename(dirpath, namebase)
    check_clobber(dotfilename, clobber)
    pngfilename = make_pngfilename(dirpath, namebase)
    check_clobber(pngfilename, clobber)
    pdffilename = make_pdffilename(dirpath, namebase)
    check_clobber(pdffilename, clobber)

    # Replace labels with mnemonic labels, if mnemonics are given
    mnemonics = map_mnemonics(labels, mnemonic_file)
    ordered_keys, labels = get_ordered_labels(labels, mnemonics)

    # Threshold
    if q_thresh > 0:
        quantile = R.call('matrix.find_quantile', probs, q_thresh)[0]
        log("Removing edges below %.4f" % float(quantile), verbose)

        probs[probs < quantile] = 0
    elif p_thresh > 0:
        log("Removing connections below %.4f" % p_thresh, verbose)

        if lenient_thresh:
            row_wise_remove = (probs < p_thresh)
            col_wise = probs / probs.sum(axis=0) # Col-normalize
            col_wise_remove = (col_wise < p_thresh)
            probs[row_wise_remove & col_wise_remove] = 0
        else:
            probs[probs < p_thresh] = 0

    # Create graph out of non-zero edges
    G = pgv.AGraph(strict=False, directed=True)

    rows, cols = where(probs > 0)

    max_val = probs.max()
    min_val = probs[probs > 0].min()
    weights = (probs - min_val)/(max_val-min_val) + BASE_WEIGHT
    weights[weights < BASE_WEIGHT] = 0
    for row, col in zip(rows, cols):
        weight = weights[row, col]
        G.add_edge(labels[row], labels[col],
                   penwidth=str(EDGE_WEIGHT * weight),
                   arrowsize=str(ARROW_WEIGHT * weight))

    G.node_attr.update(fontname=fontname)
    if gene_graph:
        G.node_attr.update(shape="plaintext")
        ps_dir = os.path.join(os.path.split(dirpath)[0], "images")
        for node in G.nodes():
            node.attr["image"] = "%s/%s.ps" % (ps_dir, str(node))
            node.attr["label"] = " "

    G.write(dotfilename)

    log("Drawing graphs...", verbose, end="")

    G.layout()

    try:
        G.draw(pngfilename)
    except:
        log("Error: Failed to draw png graph")

    try:
        # Try to generate pdf from dot
        if gene_graph:
            layout_prog = "dot"
        else:
            layout_prog = "neato"

        cmd = " ".join([layout_prog, "-Tps2", dotfilename, "|",
                        "ps2pdf", "-dAutoRotatePages=/None", "-", pdffilename])
        code = call(cmd, shell=True)
        if code != 0:
            raise Exception()
    except Exception, e:
        log("Error: Failed to draw pdf graph: %s" % str(e))

    log(" done", verbose)

## Package entry point
def validate(bedfilename, dirpath, ddgram=False, p_thresh=P_THRESH,
             q_thresh=Q_THRESH, replot=False, noplot=False, nograph=False,
             gene_graph=False, clobber=False, gmtk=False, verbose=True,
             mnemonic_file=None, ropts=None):
    setup_directory(dirpath)

    if gmtk:
        from .gmtk_parameters import load_gmtk_transitions
        labels, probs = load_gmtk_transitions(bedfilename)
        if mnemonic_file is None:
            mnemonic_file = create_mnemonic_file(bedfilename, dirpath,
                                                 clobber=clobber,
                                                 gmtk=gmtk, verbose=verbose)
    else:
        segmentation = Segmentation(bedfilename, verbose=verbose)
        assert segmentation is not None

        if replot:
            probs = load_transition_probs(dirpath)
            labels = segmentation.labels
        else:
            # Calculate transition probabilities for each label
            labels, probs = calc_transitions(segmentation)


    if not replot:
        save_tab(labels, probs, dirpath, clobber=clobber, verbose=verbose)

    if not noplot:
        with open_transcript(dirpath, MODULE) as transcriptfile:
            save_plot(dirpath, ddgram=ddgram, verbose=verbose,
                      clobber=clobber, mnemonic_file=mnemonic_file,
                      transcriptfile=transcriptfile, ropts=ropts)

    if not nograph:
        save_graph(labels, probs, dirpath, clobber=clobber,
                   p_thresh=p_thresh, q_thresh=q_thresh,
                   gene_graph=gene_graph, mnemonic_file=mnemonic_file,
                   verbose=verbose)

    save_html(dirpath, p_thresh=p_thresh, q_thresh=q_thresh,
              clobber=clobber, verbose=verbose)

def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] SEGMENTATION"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'replot', 'noplot'])
    group.add_option("--nograph", action="store_true",
                     dest="nograph", default=False,
                     help="Do not generate transition graph")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "Transition frequency plot options")
    group.add_option("--dd", "--dendrogram", dest="ddgram",
                     default=False, action="store_true",
                     help="include dendrogram along edge of levelplot"
                     " [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Transition graph options")
    group.add_option("-P", "--prob-threshold", dest="p_thresh",
                     type="float", default=P_THRESH, metavar="VAL",
                     help="ignore all transitions with probabilities below"
                     " this absolute threshold [default: %default]")
    group.add_option("-Q", "--quantile-threshold", dest="q_thresh",
                     type="float", default=Q_THRESH, metavar="VAL",
                     help="ignore transitions with probabilities below this"
                     " probability quantile [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Non-segmentation files")
    group.add_option("-g", "--gmtk", action="store_true",
                     dest="gmtk", default=False,
                     help="The SEGMENTATION argument will instead be treated"
                     " as a GMTK parameter file. If a mnemonic file is not"
                     " specified, one will be created and used.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 1:
        parser.error("Inappropriate number of arguments")

    if options.p_thresh > 0 and options.q_thresh > 0:
        parser.error("Cannot specify both absolute and quantile thresholds")
    if options.q_thresh < 0 or options.q_thresh > 1:
        parser.error("Quantile threshold should be in range [0, 1]")
    if options.p_thresh < 0 or options.p_thresh > 1:
        parser.error("Probability threshold should be in range [0, 1]")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)

    kwargs = dict(options.__dict__)
    args = [args[0], kwargs.pop('outdir')]
    validate(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

"""
gmtk_parameters.py
"""

from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE = "gmtk_parameters"

import os
import sys

from numpy import array

from . import log, die, RInterface, add_common_options
from .common import setup_directory
from .html import save_html_div
from .transition import save_plot, save_graph
from .mnemonics import create_mnemonic_file
from .signal_distribution import save_stats_plot

NAMEBASE = "%s" % MODULE
NAMEBASE_GRAPH = os.extsep.join([NAMEBASE, "graph"])
NAMEBASE_STATS = os.extsep.join([NAMEBASE, "stats"])

HTML_TEMPLATE_FILENAME = "gmtk_div.tmpl"
HTML_TITLE = "GMTK Theoretical Parameters"

P_THRESH = 0.15  # Default
Q_THRESH = 0.0

R = RInterface(["common.R", "transition.R"])

def get_default_labels(num_labels):
    """Generate default labels (0 through num_labels-1)"""
    return dict([(val, str(val)) for val in range(0, num_labels)])

def load_gmtk_transitions(gmtk_file, verbose=True):
    """Loads probabilites from a gmtk_file, through R.

    Returns probs as a numpy.array
    """
    R.start(verbose=verbose)

    r_data = R.call("read.gmtk.transition", gmtk_file)
    # Rpy automatically transposes, so need to transpose it back
    probs = array(r_data, dtype="double").transpose()
    num_labels = probs.shape[0]
    labels = get_default_labels(num_labels)

    return labels, probs

def save_html(dirpath, gmtk_file, p_thresh, q_thresh, clobber=False):
    extra_namebases = {"graph": NAMEBASE_GRAPH,
                       "stats": NAMEBASE_STATS}

    if p_thresh > 0:
        thresh = "P > %s" % p_thresh
    elif q_thresh > 0:
        thresh = "P above %.2th quantile" % q_thresh

    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, extra_namebases=extra_namebases,
                  title=HTML_TITLE, thresh=thresh,
                  gmtk_file=os.path.basename(gmtk_file))

## Package entry point
def validate(gmtk_file, dirpath, p_thresh=P_THRESH, q_thresh=Q_THRESH,
             noplot=False, nograph=False, gene_graph=False, clobber=False,
             translation_file=None, allow_regex=False, mnemonic_file=None,
             create_mnemonics=False, verbose=True, ropts=None):

    setup_directory(dirpath)

    if not os.path.isfile(gmtk_file):
        die("Could not find GMTK file: %s" % gmtk_file)

    # XXX: why does this print "getbuffer" a few times? debug R
    # read.gmtk.transition() code
    log("Loading gmtk transitions...", verbose, end="")

    labels, probs = load_gmtk_transitions(gmtk_file, verbose=verbose)
    log(" done", verbose)

    # If mnemonics weren't specified, let's create a mnemonic file
    if mnemonic_file is None and create_mnemonics:
        mnemonic_file = \
            create_mnemonic_file(gmtk_file, dirpath, verbose=verbose,
                                 clobber=clobber, gmtk=True)

    if not noplot:
        save_plot(dirpath, namebase=NAMEBASE, filename=gmtk_file,
                  verbose=verbose, clobber=clobber, gmtk=True,
                  mnemonic_file=mnemonic_file, ropts=ropts)
        save_stats_plot(dirpath, namebase=NAMEBASE_STATS, filename=gmtk_file,
                        clobber=clobber, gmtk=True, ropts=ropts,
                        mnemonic_file=mnemonic_file,
                        translation_file=translation_file,
                        allow_regex=allow_regex, verbose=verbose)

    if not nograph:
        save_graph(labels, probs, dirpath, clobber=clobber,
                   p_thresh=p_thresh, q_thresh=q_thresh,
                   gene_graph=gene_graph, mnemonic_file=mnemonic_file,
                   namebase=NAMEBASE_GRAPH, verbose=verbose)

    save_html(dirpath, gmtk_file, p_thresh=p_thresh, q_thresh=q_thresh,
              clobber=clobber)


def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] PARAMSFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'noplot'])
    group.add_option("--nograph", action="store_true",
                     dest="nograph", default=False,
                     help="Do not generate transition graph.")
    group.add_option("--create-mnemonics", action="store_true",
                     dest="create_mnemonics", default=False,
                     help="If mnemonics are not specified, they will be"
                     " created and used for plotting")
    group.add_option("--allow-regex", dest="allow_regex",
                     default=False, action="store_true",
                     help="If a file is specified with --trackname-translation"
                     ", the mappings in this file will be interpreted as"
                     " regular expressions instead of exact mappings."
                     " Thus, all underscores could be converted to periods"
                     " with the single line: `_<TAB>.`. Mappings will be"
                     " applied sequentially, so a second line of:"
                     " `\\.<TAB>#` would (together with the first line)"
                     " convert all periods and underscores to pounds.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    group.add_option("-t", "--trackname-translation", dest="translation_file",
                     default=None, metavar="FILE",
                     help="Should be a file with rows <old-trackname>"
                     "<TAB><new-trackname>. Tracknames will be translated"
                     " using this mapping before plotting the stats plot."
                     " By default, <old-trackname> must exactly match"
                     " the name of a track, but --allow-regex provides"
                     " more flexibility.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Transition graph options")
    group.add_option("-P", "--prob-threshold", dest="p_thresh",
                     type="float", default=P_THRESH, metavar="VAL",
                     help="ignore all transitions with probabilities below"
                     " this absolute threshold [default: %default]")
    group.add_option("-Q", "--quantile-threshold", dest="q_thresh",
                     type="float", default=Q_THRESH, metavar="VAL",
                     help="ignore transitions with probabilities below this"
                     " probability quantile [default: %default]")
    group.add_option("--gene-graph", dest="gene_graph",
                     default=False, action="store_true",
                     help="Make each node of the graph a reference to a .ps"
                     " image an \"image\" subdirectory. Currently, these .ps"
                     " files need to be made separately.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 1:
        parser.error("Inappropriate number of arguments")

    if options.p_thresh > 0 and options.q_thresh > 0:
        parser.error("Cannot specify both absolute and quantile thresholds")
    if options.q_thresh < 0 or options.q_thresh > 1:
        parser.error("Quantile threshold should be in range [0, 1]")
    if options.p_thresh < 0 or options.p_thresh > 1:
        parser.error("Probability threshold should be in range [0, 1]")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)

    kwargs = dict(options.__dict__)
    outdir = kwargs.pop('outdir')
    args = [args[0], outdir]
    validate(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())
